from typing import TYPE_CHECKING

if not TYPE_CHECKING:
    ...

if TYPE_CHECKING:
    ...
else:
    ...

def func(value: int | str) -> None:
    match value:
        case int(): ...
        case str(): ...
        case _ if not TYPE_CHECKING: ...



# This sample tests the handling of the dataclass_transform mechanism
# when applied to a decorator function.

from typing import Any, Callable, TypeVar, overload
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    dataclass_transform,
)

_T = TypeVar("_T")


@overload
@dataclass_transform(kw_only_default=True, order_default=True)
def create_model(cls: _T) -> _T: ...


@overload
@dataclass_transform(kw_only_default=True, order_default=True)
def create_model(
    *,
    frozen: bool = False,
    kw_only: bool = True,
    order: bool = True,
) -> Callable[[_T], _T]: ...


def create_model(*args: Any, **kwargs: Any) -> Any: ...


@create_model(kw_only=False, order=False)
class Customer1:
    id: int
    name: str


@create_model(frozen=True)
class Customer2:
    id: int
    name: str


@create_model(frozen=True)
class Customer2Subclass(Customer2):
    salary: float


c1_1 = Customer1(id=3, name="Sue")
c1_1.id = 4

c1_2 = Customer1(3, "Sue")
c1_2.name = "Susan"

# This should generate an error because of a type mismatch.
c1_2.name = 3

# This should generate an error because comparison methods are
# not synthesized.
v1 = c1_1 < c1_2

# This should generate an error because salary is not
# a defined field.
c1_3 = Customer1(id=3, name="Sue", salary=40000)

c2_1 = Customer2(id=0, name="John")

# This should generate an error because Customer2 supports
# keyword-only parameters for its constructor.
c2_2 = Customer2(0, "John")

v2 = c2_1 < c2_2


@dataclass_transform(kw_only_default=True, order_default=True, frozen_default=True)
def create_model_frozen(cls: _T) -> _T: ...


@create_model_frozen
class Customer3:
    id: int
    name: str


# This should generate an error because a non-frozen class
# cannot inherit from a frozen class.
@create_model
class Customer3Subclass(Customer3):
    age: int


c3_1 = Customer3(id=2, name="hi")

# This should generate an error because Customer3 is frozen.
c3_1.id = 4



# This sample tests the case where a function that uses a ParamSpec
# is passed to itself. This should not cause a crash or infinite recursion
# within the type evaluator.

from typing import TypeVar, Callable
from typing_extensions import ParamSpec  # pyright: ignore[reportMissingModuleSource]

P = ParamSpec("P")
R = TypeVar("R")


def func1(x: Callable[P, R]) -> Callable[P, R]:
    return x


func1(func1)



# This sample tests error cases associated with the "type" statement
# introduced in PEP 695.

from typing import Any, Callable


T1 = 0

type TA1[T1] = int

reveal_type(TA1, expected_text="TypeAliasType")


class ClassA[T2]:
    type TA2 = int
    type TA3 = str

    type TA4 = int

    T2 = 4


T2 = 4

type TA5[S1, *S2, **S3] = Callable[S3, S1] | tuple[*S2]

X1 = TA5[int, tuple[int, str], ...]

type TA6 = TA5[int, tuple[int, str], ...]

val1: TA5
val2: TA6

if 1 < 2:
    # This should generate an error because it is obscured.
    type TA7 = int
else:
    type TA7 = int


def func1() -> type[int]: ...


# This should generate an error because a call expression is not
# allowed in a type alias definition.
type TA8 = func1()

# This should generate two errors because a tuple and index expression is not
# allowed in a type alias definition.
type TA9 = (int, str, str)[0]


type TA10 = int

# This should generate an error.
TA10.bit_count(1)

# This should generate an error.
TA10(0)

list[TA10]()


# This should generate an error.
class DerivedInt(TA10):
    pass


def func2(x: object):
    # This should generate an error.
    if isinstance(x, TA10):
        reveal_type(x)


type TA11 = Callable[..., Any]


def func3(cb: TA11):
    cb()


def func4():
    # This should generate an error.
    type TA12 = int


type TA12[T] = "list[T]"
ta12: TA12[int] = [1, 2, 3]

# This should generate an error.
type TA13[T] = ...



from __future__ import annotations

import sys
from json import dumps, loads
from pathlib import Path
from typing import cast

version_file = Path(__file__).parents[2] / "packages/pyright-internal/src/version.json"


def get() -> str:
    return cast(str, loads(version_file.read_text()))


def set_version(value: str):
    _ = version_file.write_text(dumps(value))


if __name__ == "__main__":
    set_version(sys.argv[1])



from __future__ import annotations

import json
from functools import partial
from subprocess import run as stupid_run
from typing import TYPE_CHECKING

# execution environment moment
from build.py3_8.version import get  # pyright: ignore[reportMissingTypeStubs]

if TYPE_CHECKING:
    from collections.abc import Iterable

    from mkdocs_macros.plugin import MacrosPlugin

run = partial(stupid_run, check=True, capture_output=True)


def define_env(env: MacrosPlugin):
    env.macro(generate_diagnostic_rule_table)  # pyright:ignore[reportUnknownMemberType]
    env.macro(get, "basedpyright_version")  # pyright: ignore[reportUnknownMemberType]


def generate_diagnostic_rule_table() -> str:
    """generates the table of `typeCheckingMode` defaults"""
    _ = run(["npm", "run", "build:cli:dev"])
    diagnostic_rulesets: list[dict[str, bool | str]] = json.loads(
        run(["node", "packages/pyright/index.js", "--printdiagnosticrulesets"]).stdout
    )
    headers = diagnostic_rulesets[0].keys()
    result: list[Iterable[str]] = [(f"**{header}**" for header in headers), (":-" for _ in headers)]
    for row in diagnostic_rulesets:
        diagnostic_rule, *values = row.values()
        result.append([str(diagnostic_rule), *(json.dumps(value) for value in values)])
    return "\n".join("|".join(row) for row in result)



# This sample tests that any attribute that is treated as a member at
# runtime does not have a type annotation. The typing spec indicates that
# type checkers should flag such conditions as errors.

from enum import Enum
from typing import Callable, Final


class Enum1(Enum):
    # This should generate an error.
    MEMBER_1: int = 1

    # This should generate an error.
    MEMBER_2: Final = 3

    _NON_MEMBER_: int = 3

    NON_MEMBER_CALLABLE: Callable[[], int] = lambda: 1



# This sample tests the case where a generic class is passed as an
# argument to a function that accepts a generic callable parameter.
# The class-scoped TypeVars for the class must be preserved when
# solving the higher-order TypeVars.

from itertools import compress
from typing import Any, Iterable


def func1(a: Iterable[Iterable[tuple[str, int]]], b: Any) -> None:
    c = map(compress, a, b)
    reveal_type(c, expected_text="map[compress[tuple[str, int]]]")



# This sample tests the handling of __getattr__, __setattr__, and
# __delattr__ methods.

from typing import Any, Literal, TypeVar, overload

T = TypeVar("T")


class A:
    @overload
    def __getattr__(self, key: Literal["a"]) -> Literal["x"]: ...

    @overload
    def __getattr__(self, key: Literal["b"]) -> Literal[4]: ...

    @overload
    def __getattr__(self, key: Literal["c"]) -> Literal["y"]: ...

    @overload
    def __getattr__(self: T, key: Literal["d"]) -> T: ...

    def __getattr__(self, key: Literal["a", "b", "c", "d"]) -> Any: ...

    @overload
    def __setattr__(self, key: Literal["e"], val: str): ...

    @overload
    def __setattr__(self, key: Literal["f"], val: int): ...

    def __setattr__(self, key: str, val: str | int):
        pass

    @overload
    def __delattr__(self, key: Literal["g"]): ...

    @overload
    def __delattr__(self, key: Literal["h"]): ...

    def __delattr__(self, key: str):
        pass


a = A()

reveal_type(a.a, expected_text="Literal['x']")
reveal_type(a.b, expected_text="Literal[4]")
reveal_type(a.c, expected_text="Literal['y']")
reveal_type(a.d, expected_text="A")

# This should generate an error.
reveal_type(a.e)

# This should generate an error.
a.a = 4

a.e = "4"

# This should generate an error.
a.e = 4

# This should generate an error.
a.f = "4"

a.f = 4

# This should generate an error.
del a.e

del a.g

del a.h


# Test asymmetric __getattr__ and __setattr__ methods. We should not
# narrow the type on assignment in this case.
class B:
    def __setattr__(self, __name: str, __value: Any) -> None:
        pass

    def __getattr__(self, __attr: str) -> int:
        return 10


a = B()
a.test = "anything"
reveal_type(a.test, expected_text="int")



# This sample tests the type checker's ability to
# handle various class definition cases.


from typing import Any, Self, TypeVar


T = TypeVar("T")
T2 = TypeVar("T2", bound=type[Any])


class A:
    ...


class B:
    C: type[A]


app = B()


class D(app.C):
    ...


class EMeta(type):
    def __new__(mcls, *args: Any, **kwargs: Any):
        ...


class E(metaclass=EMeta):
    pass


class F(E):
    pass


class G[T](E, metaclass=type):
    def my_method(self) -> "G":
        reveal_type(__class__, expected_text="type[G[Unknown]]")
        return __class__()


# This should generate an error because only one metaclass is supported.
class H(E, metaclass=type, metaclass=type):
    pass


class I(E, other_keyword=2):
    pass


args = [1, 2, 3]
kwargs = {"foo": 5}


class J(*args, **kwargs):
    pass


def func1(x: type) -> object:
    class Y(x):
        pass

    return Y()


# This should generate an error because a TypeVar can't be used as a base class.
class K(T):
    pass


class L(type[T]):
    pass


def func2(cls: type[T]):
    class M(cls):
        pass


def func3(cls: T2) -> T2:
    class M(cls):
        pass
    
    return M



# This sample tests the reportIncompatibleVariableOverride
# configuration option in cases involving multiple inheritance
# where the override symbol is type compatible with the overridden.


# pyright: reportIncompatibleVariableOverride=true


from typing import NotRequired, Required, TypedDict


class A:
    class M:
        pass


class B0(A):
    class M(A.M):
        pass


class B1(A):
    class M(A.M):
        pass


class C(B0, B1):
    class M(B0.M, B1.M):
        pass


class D0(B0):
    pass


class D1(B1):
    pass


class D(D0, D1, C):
    pass


class E0(B0):
    pass


class E1(B1):
    pass


# This should generate an error because B0.M is not
# type compatible with B1.M.
class E(E0, E1):
    pass


class TD_A1(TypedDict):
    x: Required[int]
    y: Required[int]


class TD_A2(TypedDict):
    x: NotRequired[int]
    y: Required[int]


# This should generate an error for x but not y.
class TD_A(TD_A1, TD_A2): ...



# This sample tests various assignment scenarios where
# there is an expected type, so bidirectional type
# inference is used.

from typing import Callable, Literal, Protocol

f1: Callable[[int, int], int] = lambda a, b: a + b

# This should generate an error because x should be
# determined to be an "int", so "len(x)" is invalid.
map(lambda x: len(x), [1, 2, 3])


def must_be_int(val: int):
    return val


d1: dict[str, tuple[int, Callable[[int], int]]] = {
    "hello": (3, lambda x: must_be_int(x))
}

d2: dict[str, tuple[int, Callable[[int], int]]] = {
    # This should generate an error because the key is not a str.
    3: (3, lambda x: must_be_int(x))
}

d3: dict[str, tuple[int, Callable[[int], int]]] = {
    # This should generate an error because the first element
    # of the tuple is not the correct type.
    "3": (3.0, lambda x: must_be_int(x))
}

d4: dict[str, tuple[int, Callable[[int], int]]] = {
    # This should generate an error because the lambda
    # type doesn't match.
    "3": (3, lambda _: 3.4)
}


class Adder(Protocol):
    def __call__(self, x: int, y: dict[str, int]) -> int: ...


v1: Adder = lambda x, y: x + y["hi"]
reveal_type(v1, expected_text="(x: int, y: dict[str, int]) -> int")


class A:
    @classmethod
    def method1(cls):
        cls.v1: list[Literal[0]] = [] if issubclass(cls, int) else [0]



# This sample tests the validation of member values.

from enum import Enum


class Enum1(Enum):
    _value_: int
    RED = 1

    # This should generate an error because of a type mismatch.
    GREEN = "green"


class Enum2(Enum):
    def __new__(cls, value: int):
        obj = object.__new__(cls)
        obj._value_ = value

    RED = 1

    # This should generate an error because of a type mismatch.
    GREEN = "green"

    # This should generate an error because of a type mismatch.
    BLUE = (1, "blue")


class Enum3(Enum):
    def __init__(self, value: int):
        self._value_ = value

    RED = 1

    # This should generate an error because of a type mismatch.
    GREEN = "green"

    # This should generate an error because of a type mismatch.
    BLUE = (1, "blue")


class Enum4(Enum):
    def __init__(self, value: int, other: str):
        self._value_ = value

    # This should generate an error because of a type mismatch.
    RED = 1

    # This should generate an error because of a type mismatch.
    GREEN = "green"

    BLUE = (1, "blue")

    # This should generate an error because of a type mismatch.
    GRAY = (1, "blue", 1)



# This sample tests the case where a class derives from a named tuple.
# The synthesized __new__ method should be able to handle this.

from collections import namedtuple
from typing import NamedTuple


Class1 = namedtuple("Class1", "name")


class Class2(Class1):
    some_class_member = 1


reveal_type(Class2(name="a"), expected_text="Class2")

Class3 = NamedTuple("Class3", [("name", str)])


class Class4(Class3):
    some_class_member = 1


reveal_type(Class4(name="a"), expected_text="Class4")



# This sample tests the logic for validating that an explicit
# "self" or "cls" parameter type is honored when binding the
# method to an object or class.

from typing import Callable, Protocol, TypeVar


class HasItemProtocol1(Protocol):
    item: int


class Mixin1:
    def do_stuff(self: HasItemProtocol1):
        pass


class A1(Mixin1):
    item = 1


class B1(Mixin1):
    item = "hi"


class C1(Mixin1):
    pass


A1().do_stuff()

# This should generate an error because B1 doesn't
# match the protocol.
B1().do_stuff()

# This should generate an error because C1 doesn't
# match the protocol.
C1().do_stuff()


class HasItemProtocol2(Protocol):
    def must_have(self) -> None:
        pass


class Mixin2:
    @classmethod
    def do_stuff(cls: type[HasItemProtocol2]):
        pass


class A2(Mixin2):
    def must_have(self) -> None:
        pass


class B2(Mixin2):
    pass


A2.do_stuff()

# This should generate an error because B2 doesn't
# match the protocol.
B2.do_stuff()


class D:
    pass


class E:
    @staticmethod
    def get_or_create(context: D):
        return object.__getattribute__(context, "")


T_F = TypeVar("T_F", bound="F")


class F:
    @classmethod
    def bar(cls: type[T_F]) -> T_F: ...

    def baz(self) -> None:
        self.bar()


class ClassWithNewOverride:
    def __new__(cls):
        return object.__new__(cls)


def curry[First, *Rest, Result](
    function: Callable[[First, *Rest], Result],
) -> Callable[[*Rest], Callable[[First], Result]]: ...


class EvilProto1(Protocol):
    @curry
    def __call__[A, B](
        self, a: Callable[[A], B], b: Callable[[Callable[[A], B]], A]
    ) -> B: ...


# This should generate an error and not hang.
EvilProto1.__call__


# This should generate an error and not hang.
p: EvilProto1 = curry(lambda a, b: a(b(a)))


class G:
    type TypeAlias1 = Callable[[], None]

    def __init__(self):
        self.someVarNoArgs: G.TypeAlias1


g_ta: G.TypeAlias1



# This sample tests the case where a protocol is specialized with
# a literal type.

from typing import Any, Literal, Protocol, TypeVar


class Negatable(Protocol):
    def __neg__(self) -> "Negatable": ...


def func1(x: Negatable) -> None: ...


func1(0)


def func2(val: Literal[0, 1]):
    func1(val)


T = TypeVar("T", covariant=True)


class SupportsGetItem(Protocol[T]):
    def __getitem__(self, __k: int) -> T: ...


def func3(a: tuple[Any, ...]):
    x: SupportsGetItem[Literal["a"]] = a


def func4(x: SupportsGetItem[T]) -> T:
    return x[0]


def func5(x: list[int] | list[str]) -> None:
    y = func4(x)
    reveal_type(y, expected_text="int | str")



# This sample tests the handling of a higher-order function
# that accepts a generic function as a callback.

from typing import Callable, TypeVar

A = TypeVar("A")
B = TypeVar("B")


def func1(fn: Callable[[A, B], A], b: B) -> A: ...


def func2(a: A, x: A) -> A: ...


def func3(a: A) -> A:
    return func1(func2, a)



# This sample tests the type checker's handling of Enum.

from enum import Enum, EnumMeta, IntEnum
from typing import Self


TestEnum1 = Enum("TestEnum1", "   A   B, , ,C , \t D\t")
TestEnum2 = IntEnum("TestEnum2", "AA BB CC DD")


class TestEnum3(Enum):
    A = 0
    B = 1
    C = 2
    D = 3


a = TestEnum1["A"]
aa = TestEnum1.A
reveal_type(aa.name, expected_text="Literal['A']")
reveal_type(aa._name_, expected_text="Literal['A']")
reveal_type(aa.value, expected_text="Literal[1]")
reveal_type(aa._value_, expected_text="Literal[1]")
reveal_type(TestEnum1.D.name, expected_text="Literal['D']")
reveal_type(TestEnum1.D._name_, expected_text="Literal['D']")
reveal_type(TestEnum1.D.value, expected_text="Literal[4]")
reveal_type(TestEnum1.D._value_, expected_text="Literal[4]")


def func1(te3: TestEnum3):
    reveal_type(te3.name, expected_text="Literal['A', 'B', 'C', 'D']")
    reveal_type(te3._name_, expected_text="Literal['A', 'B', 'C', 'D']")
    reveal_type(te3.value, expected_text="Literal[0, 1, 2, 3]")
    reveal_type(te3._value_, expected_text="Literal[0, 1, 2, 3]")


reveal_type(TestEnum3.name, expected_text="property")
reveal_type(TestEnum3._name_, expected_text="str")
reveal_type(TestEnum3.value, expected_text="property")
reveal_type(TestEnum3._value_, expected_text="Any")


# This should generate an error because "Z" isn't
# a valid member.
z = TestEnum1.Z


bb = TestEnum2.BB

# This should generate an error because "A" isn't
# a valid member.
z = TestEnum2.A


b = TestEnum3.B

# This should generate an error because "Z" isn't
# a valid member.
z = TestEnum3.Z

reveal_type(TestEnum3["A"], expected_text="TestEnum3")

# Test that enum classes are iterable.
list1 = list(TestEnum3)
reveal_type(list1, expected_text="list[TestEnum3]")

list2 = [i for i in TestEnum3]
reveal_type(list2, expected_text="list[TestEnum3]")

num_items_in_enum3 = len(TestEnum3)
reveal_type(num_items_in_enum3, expected_text="int")

reveal_type(TestEnum3.A.name, expected_text="Literal['A']")
reveal_type(TestEnum3.B._name_, expected_text="Literal['B']")
reveal_type(TestEnum3.C.value, expected_text="Literal[2]")
reveal_type(TestEnum3.D._value_, expected_text="Literal[3]")


TestEnum4 = Enum("TestEnum4", ["A", "B", "C", "D"])
reveal_type(TestEnum4.A, expected_text="Literal[TestEnum4.A]")
reveal_type(TestEnum4.D, expected_text="Literal[TestEnum4.D]")
reveal_type(TestEnum4.A.name, expected_text="Literal['A']")
reveal_type(TestEnum4.B._name_, expected_text="Literal['B']")
reveal_type(TestEnum4.C.value, expected_text="Literal[3]")
reveal_type(TestEnum4.D._value_, expected_text="Literal[4]")

TestEnum5 = Enum("TestEnum5", ("A", "B", "C", "D"))
reveal_type(TestEnum5.A, expected_text="Literal[TestEnum5.A]")
reveal_type(TestEnum5.D, expected_text="Literal[TestEnum5.D]")
reveal_type(TestEnum5.A.name, expected_text="Literal['A']")
reveal_type(TestEnum5.B._name_, expected_text="Literal['B']")
reveal_type(TestEnum5.C.value, expected_text="Literal[3]")
reveal_type(TestEnum5.D._value_, expected_text="Literal[4]")

d_value = "d"

TestEnum6 = Enum("TestEnum6", [("A", 1), ("B", [1, 2]), ("C", "c"), ("D", d_value)])
reveal_type(TestEnum6.A, expected_text="Literal[TestEnum6.A]")
reveal_type(TestEnum6.D, expected_text="Literal[TestEnum6.D]")
reveal_type(TestEnum6.A.name, expected_text="Literal['A']")
reveal_type(TestEnum6.B._name_, expected_text="Literal['B']")
reveal_type(TestEnum6.A.value, expected_text="Literal[1]")
reveal_type(TestEnum6.B.value, expected_text="list[int]")
reveal_type(TestEnum6.C.value, expected_text="Literal['c']")
reveal_type(TestEnum6.D._value_, expected_text="Literal['d']")

TestEnum7 = Enum("TestEnum7", (("A", 1), ("D", "d")))
reveal_type(TestEnum7.A, expected_text="Literal[TestEnum7.A]")
reveal_type(TestEnum7.D, expected_text="Literal[TestEnum7.D]")
reveal_type(TestEnum7.A.name, expected_text="Literal['A']")
reveal_type(TestEnum7.A.value, expected_text="Literal[1]")
reveal_type(TestEnum7.D._value_, expected_text="Literal['d']")

TestEnum8 = Enum("TestEnum8", {"A": 1, "B": [1, 2], "C": "c", "D": d_value})
reveal_type(TestEnum8.A, expected_text="Literal[TestEnum8.A]")
reveal_type(TestEnum8.D, expected_text="Literal[TestEnum8.D]")
reveal_type(TestEnum8.A.name, expected_text="Literal['A']")
reveal_type(TestEnum8.B._name_, expected_text="Literal['B']")
reveal_type(TestEnum8.A.value, expected_text="Literal[1]")
reveal_type(TestEnum8.B.value, expected_text="list[int]")
reveal_type(TestEnum8.C.value, expected_text="Literal['c']")
reveal_type(TestEnum8.D._value_, expected_text="Literal['d']")


class TestEnum9(Enum):
    _other1: int
    _other2: int

    def __new__(cls, value: str, other1: int, other2: int):
        obj = object.__new__(cls)
        obj._value_ = value
        obj._other1 = other1
        obj._other2 = other2
        return obj

    A = ("a", 1, 2)
    B = ("b", 2, 3)


te9_A = TestEnum9.A
reveal_type(te9_A, expected_text="Literal[TestEnum9.A]")
reveal_type(te9_A.value, expected_text="Any")
reveal_type(te9_A._value_, expected_text="Any")
reveal_type(te9_A.name, expected_text="Literal['A']")
reveal_type(te9_A._name_, expected_text="Literal['A']")


class CustomEnumMeta1(EnumMeta):
    pass


class TestEnum10(Enum, metaclass=CustomEnumMeta1):
    A = 1
    B = 2


te10_A = TestEnum10.A
reveal_type(te10_A, expected_text="Literal[TestEnum10.A]")
reveal_type(te10_A.value, expected_text="Any")
reveal_type(te9_A._value_, expected_text="Any")
reveal_type(te9_A.name, expected_text="Literal['A']")
reveal_type(te9_A._name_, expected_text="Literal['A']")


def func2(e: type[Enum]):
    values = {v.value for v in e}
    reveal_type(values, expected_text="set[Any]")

    names = {v.name for v in e}
    reveal_type(names, expected_text="set[str]")


class TestEnum11(Enum):
    (A, B, C) = range(3)


te11_A = TestEnum11.A
reveal_type(te11_A, expected_text="Literal[TestEnum11.A]")
reveal_type(te11_A.value, expected_text="int")


def func3(self) -> None:
    pass


class TestEnum12(Enum):
    a = 1
    b = lambda self: None
    c = func3


reveal_type(TestEnum12.a, expected_text="Literal[TestEnum12.a]")
reveal_type(TestEnum12.b, expected_text="(self: Unknown) -> None")
reveal_type(TestEnum12.c, expected_text="(self: Unknown) -> None")


class TestEnum13(metaclass=CustomEnumMeta1):
    pass


TestEnum14 = TestEnum13("TestEnum14", "A, B, C")
reveal_type(TestEnum14.A, expected_text="Literal[TestEnum14.A]")


class TestEnum15(Enum):
    _value_: str
    A = 1
    B = 2

    def __init__(self, value: int):
        self._value_ = str(value)


te15_A = TestEnum15.A
reveal_type(te15_A, expected_text="Literal[TestEnum15.A]")
reveal_type(te15_A.value, expected_text="str")
reveal_type(te15_A._value_, expected_text="str")


class TestEnum16(IntEnum):
    A = 1
    B = 2
    C = 3

    D = C  # Alias for C


reveal_type(TestEnum16.D, expected_text="Literal[TestEnum16.C]")
reveal_type(TestEnum16.D.value, expected_text="Literal[3]")


class TestEnum17(IntEnum):
    def __new__(cls, val: int, doc: str) -> Self:
        obj = int.__new__(cls, val)
        obj._value_ = val
        obj.__doc__ = doc
        return obj


class TestEnum18(TestEnum17):
    A = (1, "A")
    B = (2, "B")


class TestEnum19(Enum):
    A = 1
    __B = 2


reveal_type(TestEnum19.A, expected_text="Literal[TestEnum19.A]")
reveal_type(TestEnum19.__B, expected_text="Literal[2]")


class TestEnum20(Enum):
    A = 1
    B = A + 1


reveal_type(TestEnum20.A, expected_text="Literal[TestEnum20.A]")
reveal_type(TestEnum20.A.value, expected_text="Literal[1]")
reveal_type(TestEnum20.B, expected_text="Literal[TestEnum20.B]")
reveal_type(TestEnum20.B.value, expected_text="Literal[2]")
reveal_type(TestEnum20.A.A.A, expected_text="Literal[TestEnum20.A]")
reveal_type(TestEnum20.A.B.A, expected_text="Literal[TestEnum20.A]")
reveal_type(TestEnum20.A.B, expected_text="Literal[TestEnum20.B]")


class TestEnum21Base(Enum, metaclass=CustomEnumMeta1):
    @property
    def value(self) -> str:
        return "test"


class TestEnum21(TestEnum21Base):
    A = 1


reveal_type(TestEnum21.A.value, expected_text="str")


class TestEnum22Base(Enum):
    @property
    def value(self) -> str:
        return "test"


class TestEnum22(TestEnum22Base):
    A = 1


reveal_type(TestEnum22.A.value, expected_text="str")



# This sample tests the case where a callable type within a function
# signature contains a generic return type within a union.

from typing import TypeVar
from collections.abc import Callable


T = TypeVar("T")
U = TypeVar("U")


def func1(f: Callable[[T], U | None], x: T) -> U:
    y = f(x)

    reveal_type(y, expected_text="U@func1 | None")

    if y is not None:
        reveal_type(y, expected_text="U@func1")
        return y

    raise ValueError()


def func2(x: T, f: Callable[[T], U | None]) -> U:
    def g() -> U:
        y = f(x)
        if y is not None:
            return y

        raise ValueError()

    return g()



# This sample verifies that the type checker allows access
# to instance variables provided by a metaclass.

from typing import Any


class MetaA(type):
    var0 = 3

    def __init__(cls, name, bases, dct):
        cls.var1 = "hi"


class ClassA(metaclass=MetaA):
    pass


# This should generate an error because var0 isn't
# accessible via an instance of this class.
ClassA().var0
reveal_type(ClassA.var0, expected_text="int")
ClassA.var0 = 1

reveal_type(ClassA().var1, expected_text="str")
reveal_type(ClassA.var1, expected_text="str")

ClassA.var1 = "hi"
ClassA().var1 = "hi"


class MetaB(type):
    def __setattr__(cls, key: str, value: Any) -> None: ...


class ClassB(metaclass=MetaB):
    var0: int


# This should generate an error
ClassB.var0 = ""
ClassB.var1 = ""

ClassB().var0 = 1

# This should generate an error
ClassB().var0 = ""

# This should generate an error
ClassB().var1 = ""



# This sample is a stub file used in conjunction with the protocolExplicit3.py test.

# This sample tests the logic that validates that a concrete class that
# explicitly derives from a protocol class implements the variables
# and functions defined in the protocol.

from abc import abstractmethod
from typing import ClassVar, Protocol

class Protocol1(Protocol):
    cm1: ClassVar[int]
    im1: int

class Protocol2(Protocol):
    cm10: int

class Protocol3(Protocol2, Protocol):
    pass

class Protocol5(Protocol):
    def method1(self) -> int: ...

class Protocol6(Protocol):
    x: int

class Protocol7(Protocol):
    @abstractmethod
    def method1(self): ...



# This sample tests a difficult set of circular dependencies
# between untyped variables.


class ClassA:
    def method1(self, param1):
        return ClassA()

    def method2(self):
        return {}, {}

    def method3(self, param3):
        while True:
            for key in param3.keys():
                foo1 = self.method1({key: None})
                var1, var2 = foo1.method2()

                if len(var1) < 2:
                    param3 = var2
                    break

                foo2 = foo1.method1({})
                var1, var2 = foo2.method2()
            else:
                break



# This sample tests that a dataclass member without a type annotation
# but with a field descriptor assignment results in an error.

from dataclasses import dataclass, field


@dataclass
class MyClass:
    id: int
    x: int = field()

    # This should generate an error because it will result in a runtime exception
    y = field()



# This sample tests the case where a finally clause contains some conditional
# logic that narrows the type of an expression. This narrowed type should
# persist after the finally clause.


def func1():
    file = None
    try:
        file = open("test.txt")
    except Exception:
        return None
    finally:
        if file:
            file.close()

    # This should evaluate to "TextIOWrapper", but the current
    # logic is not able to evaluate different types for file
    # based on whether it's an exception or non-exception case.
    reveal_type(file, expected_text="TextIOWrapper[_WrappedBuffer] | None")


def func2():
    file = None
    try:
        file = open("test.txt")
    except Exception:
        pass
    finally:
        if file:
            file.close()

    reveal_type(file, expected_text="TextIOWrapper[_WrappedBuffer] | None")


def func3():
    file = None
    try:
        file = open("test.txt")
    finally:
        pass

    reveal_type(file, expected_text="TextIOWrapper[_WrappedBuffer]")



# This sample tests constraint solving constrained TypeVars.

from typing import Generic, TypeVar

S = TypeVar("S", str, bytes)


def constrained(first: S, second: S) -> S:
    return first


# This should generate an error because the two arguments
# cannot satisfy the 'str' or 'bytes' constraint.
result = constrained("a", b"abc")

U = TypeVar("U", float, str)


class ClassA(Generic[U]):
    def generic_func1(self, a: U, b: str = "", **kwargs: U) -> U:
        return a


a1 = ClassA[str]()
r1 = a1.generic_func1("hi")
reveal_type(r1, expected_text="str")
r2 = a1.generic_func1("hi", test="hi")
reveal_type(r2, expected_text="str")

# This should generate an error.
r3 = a1.generic_func1("hi", test=3)
reveal_type(r3, expected_text="str")

# This should generate an error.
r4 = a1.generic_func1("hi", 3)
reveal_type(r4, expected_text="str")

a2: ClassA[int]

# This should generate an error.
a3: ClassA[Never]

ClassAAlias = ClassA[U]

# This should generate an error.
a4: ClassAAlias[Never]



# Since this module defines "overload" it is not recognized by Ruff as typing.overload
# TODO: The collections import is required, otherwise mypy crashes.
# https://github.com/python/mypy/issues/16744
import collections  # noqa: F401  # pyright: ignore[reportUnusedImport]
import sys
import typing_extensions
from _collections_abc import dict_items, dict_keys, dict_values
from _typeshed import IdentityFunction, ReadableBuffer, SupportsGetItem, SupportsGetItemViewable, SupportsKeysAndGetItem, Viewable
from abc import ABCMeta, abstractmethod
from re import Match as Match, Pattern as Pattern
from types import (
    BuiltinFunctionType,
    CodeType,
    FunctionType,
    GenericAlias,
    MethodDescriptorType,
    MethodType,
    MethodWrapperType,
    ModuleType,
    TracebackType,
    WrapperDescriptorType,
)
from typing_extensions import Never as _Never, ParamSpec as _ParamSpec, deprecated

if sys.version_info >= (3, 14):
    from _typeshed import EvaluateFunc

    from annotationlib import Format

if sys.version_info >= (3, 10):
    from types import UnionType

__all__ = [
    "AbstractSet",
    "Annotated",
    "Any",
    "AnyStr",
    "AsyncContextManager",
    "AsyncGenerator",
    "AsyncIterable",
    "AsyncIterator",
    "Awaitable",
    "BinaryIO",
    "ByteString",
    "Callable",
    "ChainMap",
    "ClassVar",
    "Collection",
    "Container",
    "ContextManager",
    "Coroutine",
    "Counter",
    "DefaultDict",
    "Deque",
    "Dict",
    "Final",
    "ForwardRef",
    "FrozenSet",
    "Generator",
    "Generic",
    "Hashable",
    "IO",
    "ItemsView",
    "Iterable",
    "Iterator",
    "KeysView",
    "List",
    "Literal",
    "Mapping",
    "MappingView",
    "Match",
    "MutableMapping",
    "MutableSequence",
    "MutableSet",
    "NamedTuple",
    "NewType",
    "NoReturn",
    "Optional",
    "OrderedDict",
    "Pattern",
    "Protocol",
    "Reversible",
    "Sequence",
    "Set",
    "Sized",
    "SupportsAbs",
    "SupportsBytes",
    "SupportsComplex",
    "SupportsFloat",
    "SupportsIndex",
    "SupportsInt",
    "SupportsRound",
    "Text",
    "TextIO",
    "Tuple",
    "Type",
    "TypeVar",
    "TypedDict",
    "Union",
    "ValuesView",
    "TYPE_CHECKING",
    "cast",
    "final",
    "get_args",
    "get_origin",
    "get_type_hints",
    "no_type_check",
    "no_type_check_decorator",
    "overload",
    "runtime_checkable",
]

if sys.version_info >= (3, 14):
    __all__ += ["evaluate_forward_ref"]

if sys.version_info >= (3, 10):
    __all__ += ["Concatenate", "ParamSpec", "ParamSpecArgs", "ParamSpecKwargs", "TypeAlias", "TypeGuard", "is_typeddict"]

if sys.version_info >= (3, 11):
    __all__ += [
        "LiteralString",
        "Never",
        "NotRequired",
        "Required",
        "Self",
        "TypeVarTuple",
        "Unpack",
        "assert_never",
        "assert_type",
        "clear_overloads",
        "dataclass_transform",
        "get_overloads",
        "reveal_type",
    ]

if sys.version_info >= (3, 12):
    __all__ += ["TypeAliasType", "override"]

if sys.version_info >= (3, 13):
    __all__ += ["get_protocol_members", "is_protocol", "NoDefault", "TypeIs", "ReadOnly"]

# We can't use this name here because it leads to issues with mypy, likely
# due to an import cycle. Below instead we use Any with a comment.
# from _typeshed import AnnotationForm

class Any:
    """
    Special type indicating an unconstrained type.

    - Any is compatible with every type.
    - Any assumed to have all methods.
    - All values assumed to be instances of Any.

    Note that all the above statements are true from the point of view of
    static type checkers. At runtime, Any should not be used with instance
    or class checks.
    """
    ...

class _Final:
    __slots__ = ("__weakref__",)

def final(f: _T) -> _T: ...
@final
class TypeVar:
    @property
    def __name__(self) -> str: ...
    @property
    def __bound__(self) -> Any | None: ...  # AnnotationForm
    @property
    def __constraints__(self) -> tuple[Any, ...]: ...  # AnnotationForm
    @property
    def __covariant__(self) -> bool: ...
    @property
    def __contravariant__(self) -> bool: ...
    if sys.version_info >= (3, 12):
        @property
        def __infer_variance__(self) -> bool: ...
    if sys.version_info >= (3, 13):
        @property
        def __default__(self) -> Any: ...  # AnnotationForm
    if sys.version_info >= (3, 13):
        def __new__(
            cls,
            name: str,
            *constraints: Any,  # AnnotationForm
            bound: Any | None = None,  # AnnotationForm
            contravariant: bool = False,
            covariant: bool = False,
            infer_variance: bool = False,
            default: Any = ...,  # AnnotationForm
        ) -> Self: ...
    elif sys.version_info >= (3, 12):
        def __new__(
            cls,
            name: str,
            *constraints: Any,  # AnnotationForm
            bound: Any | None = None,  # AnnotationForm
            covariant: bool = False,
            contravariant: bool = False,
            infer_variance: bool = False,
        ) -> Self: ...
    elif sys.version_info >= (3, 11):
        def __new__(
            cls,
            name: str,
            *constraints: Any,  # AnnotationForm
            bound: Any | None = None,  # AnnotationForm
            covariant: bool = False,
            contravariant: bool = False,
        ) -> Self: ...
    else:
        def __init__(
            self,
            name: str,
            *constraints: Any,  # AnnotationForm
            bound: Any | None = None,  # AnnotationForm
            covariant: bool = False,
            contravariant: bool = False,
        ) -> None: ...
    if sys.version_info >= (3, 10):
        def __or__(self, right: Any, /) -> _SpecialForm:
            """Return self|value."""
            ...
        def __ror__(self, left: Any, /) -> _SpecialForm:
            """Return value|self."""
            ...
    if sys.version_info >= (3, 11):
        def __typing_subst__(self, arg: Any, /) -> Any: ...
    if sys.version_info >= (3, 13):
        def __typing_prepare_subst__(self, alias: Any, args: Any, /) -> tuple[Any, ...]: ...
        def has_default(self) -> bool: ...
    if sys.version_info >= (3, 14):
        @property
        def evaluate_bound(self) -> EvaluateFunc | None: ...
        @property
        def evaluate_constraints(self) -> EvaluateFunc | None: ...
        @property
        def evaluate_default(self) -> EvaluateFunc | None: ...

# N.B. Keep this definition in sync with typing_extensions._SpecialForm
@final
class _SpecialForm(_Final):
    __slots__ = ("_name", "__doc__", "_getitem")
    def __getitem__(self, parameters: Any) -> object: ...
    if sys.version_info >= (3, 10):
        def __or__(self, other: Any) -> _SpecialForm: ...
        def __ror__(self, other: Any) -> _SpecialForm: ...

Union: _SpecialForm
Protocol: _SpecialForm
Callable: _SpecialForm
Type: _SpecialForm
NoReturn: _SpecialForm
ClassVar: _SpecialForm

Optional: _SpecialForm
Tuple: _SpecialForm
Final: _SpecialForm

Literal: _SpecialForm
TypedDict: _SpecialForm

if sys.version_info >= (3, 11):
    Self: _SpecialForm
    Never: _SpecialForm
    Unpack: _SpecialForm
    Required: _SpecialForm
    NotRequired: _SpecialForm
    LiteralString: _SpecialForm

    @final
    class TypeVarTuple:
        @property
        def __name__(self) -> str: ...
        if sys.version_info >= (3, 13):
            @property
            def __default__(self) -> Any:
                """The default value for this TypeVarTuple."""
                ...
            def has_default(self) -> bool: ...
        if sys.version_info >= (3, 13):
            def __new__(cls, name: str, *, default: Any = ...) -> Self: ...  # AnnotationForm
        elif sys.version_info >= (3, 12):
            def __new__(cls, name: str) -> Self: ...
        else:
            def __init__(self, name: str) -> None: ...

        def __iter__(self) -> Any:
            """Implement iter(self)."""
            ...
        def __typing_subst__(self, arg: Never, /) -> Never: ...
        def __typing_prepare_subst__(self, alias: Any, args: Any, /) -> tuple[Any, ...]: ...
        if sys.version_info >= (3, 14):
            @property
            def evaluate_default(self) -> EvaluateFunc | None: ...

if sys.version_info >= (3, 10):
    @final
    class ParamSpecArgs:
        @property
        def __origin__(self) -> ParamSpec: ...
        if sys.version_info >= (3, 12):
            def __new__(cls, origin: ParamSpec) -> Self: ...
        else:
            def __init__(self, origin: ParamSpec) -> None: ...

        def __eq__(self, other: object, /) -> bool:
            """Return self==value."""
            ...
        __hash__: ClassVar[None]  # type: ignore[assignment]

    @final
    class ParamSpecKwargs:
        @property
        def __origin__(self) -> ParamSpec: ...
        if sys.version_info >= (3, 12):
            def __new__(cls, origin: ParamSpec) -> Self: ...
        else:
            def __init__(self, origin: ParamSpec) -> None: ...

        def __eq__(self, other: object, /) -> bool:
            """Return self==value."""
            ...
        __hash__: ClassVar[None]  # type: ignore[assignment]

    @final
    class ParamSpec:
        @property
        def __name__(self) -> str: ...
        @property
        def __bound__(self) -> Any | None: ...  # AnnotationForm
        @property
        def __covariant__(self) -> bool: ...
        @property
        def __contravariant__(self) -> bool: ...
        if sys.version_info >= (3, 12):
            @property
            def __infer_variance__(self) -> bool: ...
        if sys.version_info >= (3, 13):
            @property
            def __default__(self) -> Any:
                """The default value for this ParamSpec."""
                ...
        if sys.version_info >= (3, 13):
            def __new__(
                cls,
                name: str,
                *,
                bound: Any | None = None,  # AnnotationForm
                contravariant: bool = False,
                covariant: bool = False,
                infer_variance: bool = False,
                default: Any = ...,  # AnnotationForm
            ) -> Self: ...
        elif sys.version_info >= (3, 12):
            def __new__(
                cls,
                name: str,
                *,
                bound: Any | None = None,  # AnnotationForm
                contravariant: bool = False,
                covariant: bool = False,
                infer_variance: bool = False,
            ) -> Self: ...
        elif sys.version_info >= (3, 11):
            def __new__(
                cls,
                name: str,
                *,
                bound: Any | None = None,  # AnnotationForm
                contravariant: bool = False,
                covariant: bool = False,
            ) -> Self: ...
        else:
            def __init__(
                self,
                name: str,
                *,
                bound: Any | None = None,  # AnnotationForm
                contravariant: bool = False,
                covariant: bool = False,
            ) -> None: ...

        @property
        def args(self) -> ParamSpecArgs:
            """Represents positional arguments."""
            ...
        @property
        def kwargs(self) -> ParamSpecKwargs:
            """Represents keyword arguments."""
            ...
        if sys.version_info >= (3, 11):
            def __typing_subst__(self, arg: Any, /) -> Any: ...
            def __typing_prepare_subst__(self, alias: Any, args: Any, /) -> tuple[Any, ...]: ...

        def __or__(self, right: Any, /) -> _SpecialForm:
            """Return self|value."""
            ...
        def __ror__(self, left: Any, /) -> _SpecialForm:
            """Return value|self."""
            ...
        if sys.version_info >= (3, 13):
            def has_default(self) -> bool: ...
        if sys.version_info >= (3, 14):
            @property
            def evaluate_default(self) -> EvaluateFunc | None: ...

    Concatenate: _SpecialForm
    TypeAlias: _SpecialForm
    TypeGuard: _SpecialForm

    class NewType:
        def __init__(self, name: str, tp: Any) -> None: ...  # AnnotationForm
        if sys.version_info >= (3, 11):
            @staticmethod
            def __call__(x: _T, /) -> _T: ...
        else:
            def __call__(self, x: _T) -> _T: ...

        def __or__(self, other: Any) -> _SpecialForm: ...
        def __ror__(self, other: Any) -> _SpecialForm: ...
        __supertype__: type | NewType
        __name__: str

else:
    def NewType(name: str, tp: Any) -> Any: ...

_F = TypeVar("_F", bound=Callable[..., Any])
_P = _ParamSpec("_P")
_T = TypeVar("_T")

_FT = TypeVar("_FT", bound=Callable[..., Any] | type)

# These type variables are used by the container types.
_S = TypeVar("_S")
_KT = TypeVar("_KT")  # Key type.
_VT = TypeVar("_VT")  # Value type.
_T_co = TypeVar("_T_co", covariant=True)  # Any type covariant containers.
_KT_co = TypeVar("_KT_co", covariant=True)  # Key type covariant containers.
_VT_co = TypeVar("_VT_co", covariant=True)  # Value type covariant containers.
_TC = TypeVar("_TC", bound=type[object])

def overload(func: _F) -> _F: ...
def no_type_check(arg: _F) -> _F: ...

if sys.version_info >= (3, 13):
    @deprecated("Deprecated since Python 3.13; removed in Python 3.15.")
    def no_type_check_decorator(decorator: Callable[_P, _T]) -> Callable[_P, _T]: ...

else:
    def no_type_check_decorator(decorator: Callable[_P, _T]) -> Callable[_P, _T]: ...

# This itself is only available during type checking
def type_check_only(func_or_cls: _FT) -> _FT: ...

# Type aliases and type constructors

@type_check_only
class _Alias:
    # Class for defining generic aliases for library types.
    def __getitem__(self, typeargs: Any) -> Any: ...

List = _Alias()
Dict = _Alias()
DefaultDict = _Alias()
Set = _Alias()
FrozenSet = _Alias()
Counter = _Alias()
Deque = _Alias()
ChainMap = _Alias()

OrderedDict = _Alias()

Annotated: _SpecialForm

# Predefined type variables.
AnyStr = TypeVar("AnyStr", str, bytes)  # noqa: Y001

@type_check_only
class _Generic:
    if sys.version_info < (3, 12):
        __slots__ = ()

    if sys.version_info >= (3, 10):
        @classmethod
        def __class_getitem__(cls, args: TypeVar | ParamSpec | tuple[TypeVar | ParamSpec, ...]) -> _Final: ...
    else:
        @classmethod
        def __class_getitem__(cls, args: TypeVar | tuple[TypeVar, ...]) -> _Final: ...

Generic: type[_Generic]

class _ProtocolMeta(ABCMeta):
    if sys.version_info >= (3, 12):
        def __init__(cls, *args: Any, **kwargs: Any) -> None: ...

# Abstract base classes.

def runtime_checkable(cls: _TC) -> _TC: ...
@runtime_checkable
class SupportsInt(Protocol, metaclass=ABCMeta):
    __slots__ = ()
    @abstractmethod
    def __int__(self) -> int: ...

@runtime_checkable
class SupportsFloat(Protocol, metaclass=ABCMeta):
    __slots__ = ()
    @abstractmethod
    def __float__(self) -> float: ...

@runtime_checkable
class SupportsComplex(Protocol, metaclass=ABCMeta):
    __slots__ = ()
    @abstractmethod
    def __complex__(self) -> complex: ...

@runtime_checkable
class SupportsBytes(Protocol, metaclass=ABCMeta):
    __slots__ = ()
    @abstractmethod
    def __bytes__(self) -> bytes: ...

@runtime_checkable
class SupportsIndex(Protocol, metaclass=ABCMeta):
    __slots__ = ()
    @abstractmethod
    def __index__(self) -> int: ...

@runtime_checkable
class SupportsAbs(Protocol[_T_co]):
    __slots__ = ()
    @abstractmethod
    def __abs__(self) -> _T_co: ...

@runtime_checkable
class SupportsRound(Protocol[_T_co]):
    __slots__ = ()
    @overload
    @abstractmethod
    def __round__(self) -> int: ...
    @overload
    @abstractmethod
    def __round__(self, ndigits: int, /) -> _T_co: ...

@runtime_checkable
class Sized(Protocol, metaclass=ABCMeta):
    """Deprecated alias to collections.abc.Sized."""
    @abstractmethod
    def __len__(self) -> int: ...

@runtime_checkable
class Hashable(Protocol, metaclass=ABCMeta):
    """Deprecated alias to collections.abc.Hashable."""
    # TODO: This is special, in that a subclass of a hashable class may not be hashable
    #   (for example, list vs. object). It's not obvious how to represent this. This class
    #   is currently mostly useless for static checking.
    @abstractmethod
    def __hash__(self) -> int:
        """Return hash(self)."""
        ...

@runtime_checkable
class Iterable(Protocol[_T_co]):
    """Deprecated alias to collections.abc.Iterable."""
    @abstractmethod
    def __iter__(self) -> Iterator[_T_co]: ...

@runtime_checkable
class Iterator(Iterable[_T_co], Protocol[_T_co]):
    """Deprecated alias to collections.abc.Iterator."""
    @abstractmethod
    def __next__(self) -> _T_co: ...
    def __iter__(self) -> Iterator[_T_co]: ...

@runtime_checkable
class Reversible(Iterable[_T_co], Protocol[_T_co]):
    """Deprecated alias to collections.abc.Reversible."""
    @abstractmethod
    def __reversed__(self) -> Iterator[_T_co]: ...

_YieldT_co = TypeVar("_YieldT_co", covariant=True)
_SendT_contra = TypeVar("_SendT_contra", contravariant=True, default=None)
_ReturnT_co = TypeVar("_ReturnT_co", covariant=True, default=None)

@runtime_checkable
class Generator(Iterator[_YieldT_co], Protocol[_YieldT_co, _SendT_contra, _ReturnT_co]):
    """Deprecated alias to collections.abc.Generator."""
    def __next__(self) -> _YieldT_co: ...
    @abstractmethod
    def send(self, value: _SendT_contra, /) -> _YieldT_co: ...
    @overload
    @abstractmethod
    def throw(
        self, typ: type[BaseException], val: BaseException | object = None, tb: TracebackType | None = None, /
    ) -> _YieldT_co: ...
    @overload
    @abstractmethod
    def throw(self, typ: BaseException, val: None = None, tb: TracebackType | None = None, /) -> _YieldT_co: ...
    if sys.version_info >= (3, 13):
        def close(self) -> _ReturnT_co | None: ...
    else:
        def close(self) -> None: ...

    def __iter__(self) -> Generator[_YieldT_co, _SendT_contra, _ReturnT_co]: ...

# NOTE: Prior to Python 3.13 these aliases are lacking the second _ExitT_co parameter
if sys.version_info >= (3, 13):
    from contextlib import AbstractAsyncContextManager as AsyncContextManager, AbstractContextManager as ContextManager
else:
    from contextlib import AbstractAsyncContextManager, AbstractContextManager

    @runtime_checkable
    class ContextManager(AbstractContextManager[_T_co, bool | None], Protocol[_T_co]):
        """A generic version of contextlib.AbstractContextManager."""
        ...

    @runtime_checkable
    class AsyncContextManager(AbstractAsyncContextManager[_T_co, bool | None], Protocol[_T_co]):
        """A generic version of contextlib.AbstractAsyncContextManager."""
        ...

@runtime_checkable
class Awaitable(Protocol[_T_co]):
    """Deprecated alias to collections.abc.Awaitable."""
    @abstractmethod
    def __await__(self) -> Generator[Any, Any, _T_co]: ...

# Non-default variations to accommodate coroutines, and `AwaitableGenerator` having a 4th type parameter.
_SendT_nd_contra = TypeVar("_SendT_nd_contra", contravariant=True)
_ReturnT_nd_co = TypeVar("_ReturnT_nd_co", covariant=True)

class Coroutine(Awaitable[_ReturnT_nd_co], Generic[_YieldT_co, _SendT_nd_contra, _ReturnT_nd_co]):
    """Deprecated alias to collections.abc.Coroutine."""
    __name__: str
    __qualname__: str

    @abstractmethod
    def send(self, value: _SendT_nd_contra, /) -> _YieldT_co: ...
    @overload
    @abstractmethod
    def throw(
        self, typ: type[BaseException], val: BaseException | object = None, tb: TracebackType | None = None, /
    ) -> _YieldT_co: ...
    @overload
    @abstractmethod
    def throw(self, typ: BaseException, val: None = None, tb: TracebackType | None = None, /) -> _YieldT_co: ...
    @abstractmethod
    def close(self) -> None: ...

# NOTE: This type does not exist in typing.py or PEP 484 but mypy needs it to exist.
# The parameters correspond to Generator, but the 4th is the original type.
# Obsolete, use _typeshed._type_checker_internals.AwaitableGenerator instead.
@type_check_only
class AwaitableGenerator(
    Awaitable[_ReturnT_nd_co],
    Generator[_YieldT_co, _SendT_nd_contra, _ReturnT_nd_co],
    Generic[_YieldT_co, _SendT_nd_contra, _ReturnT_nd_co, _S],
    metaclass=ABCMeta,
): ...

@runtime_checkable
class AsyncIterable(Protocol[_T_co]):
    """Deprecated alias to collections.abc.AsyncIterable."""
    @abstractmethod
    def __aiter__(self) -> AsyncIterator[_T_co]: ...

@runtime_checkable
class AsyncIterator(AsyncIterable[_T_co], Protocol[_T_co]):
    """Deprecated alias to collections.abc.AsyncIterator."""
    @abstractmethod
    def __anext__(self) -> Awaitable[_T_co]: ...
    def __aiter__(self) -> AsyncIterator[_T_co]: ...

@runtime_checkable
class AsyncGenerator(AsyncIterator[_YieldT_co], Protocol[_YieldT_co, _SendT_contra]):
    """Deprecated alias to collections.abc.AsyncGenerator."""
    def __anext__(self) -> Coroutine[Any, Any, _YieldT_co]: ...
    @abstractmethod
    def asend(self, value: _SendT_contra, /) -> Coroutine[Any, Any, _YieldT_co]: ...
    @overload
    @abstractmethod
    def athrow(
        self, typ: type[BaseException], val: BaseException | object = None, tb: TracebackType | None = None, /
    ) -> Coroutine[Any, Any, _YieldT_co]: ...
    @overload
    @abstractmethod
    def athrow(
        self, typ: BaseException, val: None = None, tb: TracebackType | None = None, /
    ) -> Coroutine[Any, Any, _YieldT_co]: ...
    def aclose(self) -> Coroutine[Any, Any, None]: ...

@runtime_checkable
class Container(Protocol[_T_co]):
    """Deprecated alias to collections.abc.Container."""
    # This is generic more on vibes than anything else
    @abstractmethod
    def __contains__(self, x: object, /) -> bool: ...

@runtime_checkable
class Collection(Iterable[_T_co], Container[_T_co], Protocol[_T_co]):
    """Deprecated alias to collections.abc.Collection."""
    # Implement Sized (but don't have it as a base class).
    @abstractmethod
    def __len__(self) -> int: ...

class Sequence(Reversible[_T_co], Collection[_T_co]):
    """Deprecated alias to collections.abc.Sequence."""
    @overload
    @abstractmethod
    def __getitem__(self, index: int) -> _T_co: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: slice) -> Sequence[_T_co]: ...
    # Mixin methods
    def index(self, value: Any, start: int = 0, stop: int = ...) -> int: ...
    def count(self, value: Any) -> int: ...
    def __contains__(self, value: object) -> bool: ...
    def __iter__(self) -> Iterator[_T_co]: ...
    def __reversed__(self) -> Iterator[_T_co]: ...

class MutableSequence(Sequence[_T]):
    """Deprecated alias to collections.abc.MutableSequence."""
    @abstractmethod
    def insert(self, index: int, value: _T) -> None: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: int) -> _T: ...
    @overload
    @abstractmethod
    def __getitem__(self, index: slice) -> MutableSequence[_T]: ...
    @overload
    @abstractmethod
    def __setitem__(self, index: int, value: _T) -> None: ...
    @overload
    @abstractmethod
    def __setitem__(self, index: slice, value: Iterable[_T]) -> None: ...
    @overload
    @abstractmethod
    def __delitem__(self, index: int) -> None: ...
    @overload
    @abstractmethod
    def __delitem__(self, index: slice) -> None: ...
    # Mixin methods
    def append(self, value: _T) -> None: ...
    def clear(self) -> None: ...
    def extend(self, values: Iterable[_T]) -> None: ...
    def reverse(self) -> None: ...
    def pop(self, index: int = -1) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def __iadd__(self, values: Iterable[_T]) -> typing_extensions.Self: ...

class AbstractSet(Collection[_T_co]):
    """Deprecated alias to collections.abc.Set."""
    @abstractmethod
    def __contains__(self, x: object) -> bool: ...
    def _hash(self) -> int: ...
    # Mixin methods
    def __le__(self, other: AbstractSet[Any]) -> bool:
        """Return self<=value."""
        ...
    def __lt__(self, other: AbstractSet[Any]) -> bool:
        """Return self<value."""
        ...
    def __gt__(self, other: AbstractSet[Any]) -> bool:
        """Return self>value."""
        ...
    def __ge__(self, other: AbstractSet[Any]) -> bool:
        """Return self>=value."""
        ...
    def __and__(self, other: AbstractSet[Any]) -> AbstractSet[_T_co]: ...
    def __or__(self, other: AbstractSet[_T]) -> AbstractSet[_T_co | _T]: ...
    def __sub__(self, other: AbstractSet[Any]) -> AbstractSet[_T_co]: ...
    def __xor__(self, other: AbstractSet[_T]) -> AbstractSet[_T_co | _T]: ...
    def __eq__(self, other: object) -> bool:
        """Return self==value."""
        ...
    def isdisjoint(self, other: Iterable[Any]) -> bool: ...

class MutableSet(AbstractSet[_T]):
    """Deprecated alias to collections.abc.MutableSet."""
    @abstractmethod
    def add(self, value: _T) -> None: ...
    @abstractmethod
    def discard(self, value: _T) -> None: ...
    # Mixin methods
    def clear(self) -> None: ...
    def pop(self) -> _T: ...
    def remove(self, value: _T) -> None: ...
    def __ior__(self, it: AbstractSet[_T]) -> typing_extensions.Self: ...  # type: ignore[override,misc]
    def __iand__(self, it: AbstractSet[Any]) -> typing_extensions.Self: ...
    def __ixor__(self, it: AbstractSet[_T]) -> typing_extensions.Self: ...  # type: ignore[override,misc]
    def __isub__(self, it: AbstractSet[Any]) -> typing_extensions.Self: ...

class MappingView(Sized):
    """Deprecated alias to collections.abc.MappingView."""
    __slots__ = ("_mapping",)
    def __init__(self, mapping: Sized) -> None: ...  # undocumented
    def __len__(self) -> int: ...

class ItemsView(MappingView, AbstractSet[tuple[_KT_co, _VT_co]], Generic[_KT_co, _VT_co]):
    """Deprecated alias to collections.abc.ItemsView."""
    def __init__(self, mapping: SupportsGetItemViewable[_KT_co, _VT_co]) -> None: ...  # undocumented
    def __and__(self, other: Iterable[Any]) -> set[tuple[_KT_co, _VT_co]]: ...
    def __rand__(self, other: Iterable[_T]) -> set[_T]: ...
    def __contains__(self, item: tuple[object, object]) -> bool: ...  # type: ignore[override]
    def __iter__(self) -> Iterator[tuple[_KT_co, _VT_co]]: ...
    def __or__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __ror__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __sub__(self, other: Iterable[Any]) -> set[tuple[_KT_co, _VT_co]]: ...
    def __rsub__(self, other: Iterable[_T]) -> set[_T]: ...
    def __xor__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...
    def __rxor__(self, other: Iterable[_T]) -> set[tuple[_KT_co, _VT_co] | _T]: ...

class KeysView(MappingView, AbstractSet[_KT_co]):
    """Deprecated alias to collections.abc.KeysView."""
    def __init__(self, mapping: Viewable[_KT_co]) -> None: ...  # undocumented
    def __and__(self, other: Iterable[Any]) -> set[_KT_co]: ...
    def __rand__(self, other: Iterable[_T]) -> set[_T]: ...
    def __contains__(self, key: object) -> bool: ...
    def __iter__(self) -> Iterator[_KT_co]: ...
    def __or__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __ror__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __sub__(self, other: Iterable[Any]) -> set[_KT_co]: ...
    def __rsub__(self, other: Iterable[_T]) -> set[_T]: ...
    def __xor__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...
    def __rxor__(self, other: Iterable[_T]) -> set[_KT_co | _T]: ...

class ValuesView(MappingView, Collection[_VT_co]):
    """Deprecated alias to collections.abc.ValuesView."""
    def __init__(self, mapping: SupportsGetItemViewable[Any, _VT_co]) -> None: ...  # undocumented
    def __contains__(self, value: object) -> bool: ...
    def __iter__(self) -> Iterator[_VT_co]: ...

# note for Mapping.get and MutableMapping.pop and MutableMapping.setdefault
# In _collections_abc.py the parameters are positional-or-keyword,
# but dict and types.MappingProxyType (the vast majority of Mapping types)
# don't allow keyword arguments.

class Mapping(Collection[_KT], Generic[_KT, _VT_co]):
    """Deprecated alias to collections.abc.Mapping."""
    # TODO: We wish the key type could also be covariant, but that doesn't work,
    # see discussion in https://github.com/python/typing/pull/273.
    @abstractmethod
    def __getitem__(self, key: _KT, /) -> _VT_co: ...
    # Mixin methods
    @overload
    def get(self, key: _KT, /) -> _VT_co | None: ...
    @overload
    def get(self, key: _KT, default: _VT_co, /) -> _VT_co: ...  # type: ignore[misc] # pyright: ignore[reportGeneralTypeIssues] # Covariant type as parameter
    @overload
    def get(self, key: _KT, default: _T, /) -> _VT_co | _T: ...
    def items(self) -> ItemsView[_KT, _VT_co]: ...
    def keys(self) -> KeysView[_KT]: ...
    def values(self) -> ValuesView[_VT_co]: ...
    def __contains__(self, key: object, /) -> bool: ...
    def __eq__(self, other: object, /) -> bool:
        """Return self==value."""
        ...

class MutableMapping(Mapping[_KT, _VT]):
    """Deprecated alias to collections.abc.MutableMapping."""
    @abstractmethod
    def __setitem__(self, key: _KT, value: _VT, /) -> None: ...
    @abstractmethod
    def __delitem__(self, key: _KT, /) -> None: ...
    def clear(self) -> None: ...
    @overload
    def pop(self, key: _KT, /) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _VT, /) -> _VT: ...
    @overload
    def pop(self, key: _KT, default: _T, /) -> _VT | _T: ...
    def popitem(self) -> tuple[_KT, _VT]: ...
    # This overload should be allowed only if the value type is compatible with None.
    #
    # Keep the following methods in line with MutableMapping.setdefault, modulo positional-only differences:
    # -- collections.OrderedDict.setdefault
    # -- collections.ChainMap.setdefault
    # -- weakref.WeakKeyDictionary.setdefault
    @overload
    def setdefault(self: MutableMapping[_KT, _T | None], key: _KT, default: None = None, /) -> _T | None: ...
    @overload
    def setdefault(self, key: _KT, default: _VT, /) -> _VT: ...
    # 'update' used to take a Union, but using overloading is better.
    # The second overloaded type here is a bit too general, because
    # Mapping[tuple[_KT, _VT], W] is a subclass of Iterable[tuple[_KT, _VT]],
    # but will always have the behavior of the first overloaded type
    # at runtime, leading to keys of a mix of types _KT and tuple[_KT, _VT].
    # We don't currently have any way of forcing all Mappings to use
    # the first overload, but by using overloading rather than a Union,
    # mypy will commit to using the first overload when the argument is
    # known to be a Mapping with unknown type parameters, which is closer
    # to the behavior we want. See mypy issue  #1430.
    #
    # Various mapping classes have __ior__ methods that should be kept roughly in line with .update():
    # -- dict.__ior__
    # -- os._Environ.__ior__
    # -- collections.UserDict.__ior__
    # -- collections.ChainMap.__ior__
    # -- peewee.attrdict.__add__
    # -- peewee.attrdict.__iadd__
    # -- weakref.WeakValueDictionary.__ior__
    # -- weakref.WeakKeyDictionary.__ior__
    @overload
    def update(self, m: SupportsKeysAndGetItem[_KT, _VT], /) -> None: ...
    @overload
    def update(self: SupportsGetItem[str, _VT], m: SupportsKeysAndGetItem[str, _VT], /, **kwargs: _VT) -> None: ...
    @overload
    def update(self, m: Iterable[tuple[_KT, _VT]], /) -> None: ...
    @overload
    def update(self: SupportsGetItem[str, _VT], m: Iterable[tuple[str, _VT]], /, **kwargs: _VT) -> None: ...
    @overload
    def update(self: SupportsGetItem[str, _VT], **kwargs: _VT) -> None: ...

Text = str

TYPE_CHECKING: Final[bool]

# In stubs, the arguments of the IO class are marked as positional-only.
# This differs from runtime, but better reflects the fact that in reality
# classes deriving from IO use different names for the arguments.
class IO(Generic[AnyStr]):
    # At runtime these are all abstract properties,
    # but making them abstract in the stub is hugely disruptive, for not much gain.
    # See #8726
    __slots__ = ()
    @property
    def mode(self) -> str: ...
    # Usually str, but may be bytes if a bytes path was passed to open(). See #10737.
    # If PEP 696 becomes available, we may want to use a defaulted TypeVar here.
    @property
    def name(self) -> str | Any: ...
    @abstractmethod
    def close(self) -> None: ...
    @property
    def closed(self) -> bool: ...
    @abstractmethod
    def fileno(self) -> int: ...
    @abstractmethod
    def flush(self) -> None: ...
    @abstractmethod
    def isatty(self) -> bool: ...
    @abstractmethod
    def read(self, n: int = -1, /) -> AnyStr: ...
    @abstractmethod
    def readable(self) -> bool: ...
    @abstractmethod
    def readline(self, limit: int = -1, /) -> AnyStr: ...
    @abstractmethod
    def readlines(self, hint: int = -1, /) -> list[AnyStr]: ...
    @abstractmethod
    def seek(self, offset: int, whence: int = 0, /) -> int: ...
    @abstractmethod
    def seekable(self) -> bool: ...
    @abstractmethod
    def tell(self) -> int: ...
    @abstractmethod
    def truncate(self, size: int | None = None, /) -> int: ...
    @abstractmethod
    def writable(self) -> bool: ...
    @abstractmethod
    @overload
    def write(self: IO[bytes], s: ReadableBuffer, /) -> int: ...
    @abstractmethod
    @overload
    def write(self, s: AnyStr, /) -> int: ...
    @abstractmethod
    @overload
    def writelines(self: IO[bytes], lines: Iterable[ReadableBuffer], /) -> None: ...
    @abstractmethod
    @overload
    def writelines(self, lines: Iterable[AnyStr], /) -> None: ...
    @abstractmethod
    def __next__(self) -> AnyStr: ...
    @abstractmethod
    def __iter__(self) -> Iterator[AnyStr]: ...
    @abstractmethod
    def __enter__(self) -> IO[AnyStr]: ...
    @abstractmethod
    def __exit__(
        self, type: type[BaseException] | None, value: BaseException | None, traceback: TracebackType | None, /
    ) -> None: ...

class BinaryIO(IO[bytes]):
    __slots__ = ()
    @abstractmethod
    def __enter__(self) -> BinaryIO: ...

class TextIO(IO[str]):
    # See comment regarding the @properties in the `IO` class
    __slots__ = ()
    @property
    def buffer(self) -> BinaryIO: ...
    @property
    def encoding(self) -> str: ...
    @property
    def errors(self) -> str | None: ...
    @property
    def line_buffering(self) -> int: ...  # int on PyPy, bool on CPython
    @property
    def newlines(self) -> Any: ...  # None, str or tuple
    @abstractmethod
    def __enter__(self) -> TextIO: ...

ByteString: typing_extensions.TypeAlias = bytes | bytearray | memoryview

# Functions

_get_type_hints_obj_allowed_types: typing_extensions.TypeAlias = (  # noqa: Y042
    object
    | Callable[..., Any]
    | FunctionType
    | BuiltinFunctionType
    | MethodType
    | ModuleType
    | WrapperDescriptorType
    | MethodWrapperType
    | MethodDescriptorType
)

if sys.version_info >= (3, 14):
    def get_type_hints(
        obj: _get_type_hints_obj_allowed_types,
        globalns: dict[str, Any] | None = None,
        localns: Mapping[str, Any] | None = None,
        include_extras: bool = False,
        *,
        format: Format | None = None,
    ) -> dict[str, Any]: ...  # AnnotationForm

else:
    def get_type_hints(
        obj: _get_type_hints_obj_allowed_types,
        globalns: dict[str, Any] | None = None,
        localns: Mapping[str, Any] | None = None,
        include_extras: bool = False,
    ) -> dict[str, Any]: ...  # AnnotationForm

def get_args(tp: Any) -> tuple[Any, ...]: ...  # AnnotationForm

if sys.version_info >= (3, 10):
    @overload
    def get_origin(tp: ParamSpecArgs | ParamSpecKwargs) -> ParamSpec: ...
    @overload
    def get_origin(tp: UnionType) -> type[UnionType]: ...

@overload
def get_origin(tp: GenericAlias) -> type: ...
@overload
def get_origin(tp: Any) -> Any | None: ...  # AnnotationForm
@overload
def cast(typ: type[_T], val: Any) -> _T: ...
@overload
def cast(typ: str, val: Any) -> Any: ...
@overload
def cast(typ: object, val: Any) -> Any: ...

if sys.version_info >= (3, 11):
    def reveal_type(obj: _T, /) -> _T: ...
    def assert_never(arg: Never, /) -> Never: ...
    def assert_type(val: _T, typ: Any, /) -> _T: ...  # AnnotationForm
    def clear_overloads() -> None: ...
    def get_overloads(func: Callable[..., object]) -> Sequence[Callable[..., object]]: ...
    def dataclass_transform(
        *,
        eq_default: bool = True,
        order_default: bool = False,
        kw_only_default: bool = False,
        frozen_default: bool = False,  # on 3.11, runtime accepts it as part of kwargs
        field_specifiers: tuple[type[Any] | Callable[..., Any], ...] = (),
        skip_replace: bool = False,
        **kwargs: Any,
    ) -> IdentityFunction: ...

# Type constructors

# Obsolete, will be changed to a function. Use _typeshed._type_checker_internals.NamedTupleFallback instead.
class NamedTuple(tuple[Any, ...]):
    _field_defaults: ClassVar[dict[str, Any]]
    _fields: ClassVar[tuple[str, ...]]
    # __orig_bases__ sometimes exists on <3.12, but not consistently
    # So we only add it to the stub on 3.12+.
    if sys.version_info >= (3, 12):
        __orig_bases__: ClassVar[tuple[Any, ...]]

    @overload
    def __init__(self, typename: str, fields: Iterable[tuple[str, Any]], /) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
        ...
    @overload
    @deprecated("Creating a typing.NamedTuple using keyword arguments is deprecated and support will be removed in Python 3.15")
    def __init__(self, typename: str, fields: None = None, /, **kwargs: Any) -> None:
        """Initialize self.  See help(type(self)) for accurate signature."""
        ...
    @classmethod
    def _make(cls, iterable: Iterable[Any]) -> typing_extensions.Self: ...
    def _asdict(self) -> dict[str, Any]: ...
    def _replace(self, **kwargs: Any) -> typing_extensions.Self: ...
    if sys.version_info >= (3, 13):
        def __replace__(self, **kwargs: Any) -> typing_extensions.Self: ...

# Internal mypy fallback type for all typed dicts (does not exist at runtime)
# N.B. Keep this mostly in sync with typing_extensions._TypedDict/mypy_extensions._TypedDict
# Obsolete, use _typeshed._type_checker_internals.TypedDictFallback instead.
@type_check_only
class _TypedDict(Mapping[str, object], metaclass=ABCMeta):
    __total__: ClassVar[bool]
    __required_keys__: ClassVar[frozenset[str]]
    __optional_keys__: ClassVar[frozenset[str]]
    # __orig_bases__ sometimes exists on <3.12, but not consistently,
    # so we only add it to the stub on 3.12+
    if sys.version_info >= (3, 12):
        __orig_bases__: ClassVar[tuple[Any, ...]]
    if sys.version_info >= (3, 13):
        __readonly_keys__: ClassVar[frozenset[str]]
        __mutable_keys__: ClassVar[frozenset[str]]

    def copy(self) -> typing_extensions.Self: ...
    # Using Never so that only calls using mypy plugin hook that specialize the signature
    # can go through.
    def setdefault(self, k: _Never, default: object) -> object: ...
    # Mypy plugin hook for 'pop' expects that 'default' has a type variable type.
    def pop(self, k: _Never, default: _T = ...) -> object: ...  # pyright: ignore[reportInvalidTypeVarUse]
    def update(self, m: typing_extensions.Self, /) -> None: ...
    def __delitem__(self, k: _Never) -> None: ...
    def items(self) -> dict_items[str, object]: ...
    def keys(self) -> dict_keys[str, object]: ...
    def values(self) -> dict_values[str, object]: ...
    @overload
    def __or__(self, value: typing_extensions.Self, /) -> typing_extensions.Self:
        """Return self|value."""
        ...
    @overload
    def __or__(self, value: dict[str, Any], /) -> dict[str, object]:
        """Return self|value."""
        ...
    @overload
    def __ror__(self, value: typing_extensions.Self, /) -> typing_extensions.Self:
        """Return value|self."""
        ...
    @overload
    def __ror__(self, value: dict[str, Any], /) -> dict[str, object]:
        """Return value|self."""
        ...
    # supposedly incompatible definitions of __or__ and __ior__
    def __ior__(self, value: typing_extensions.Self, /) -> typing_extensions.Self: ...  # type: ignore[misc]

if sys.version_info >= (3, 14):
    from annotationlib import ForwardRef as ForwardRef

    def evaluate_forward_ref(
        forward_ref: ForwardRef,
        *,
        owner: object = None,
        globals: dict[str, Any] | None = None,
        locals: Mapping[str, Any] | None = None,
        type_params: tuple[TypeVar, ParamSpec, TypeVarTuple] | None = None,
        format: Format | None = None,
    ) -> Any: ...  # AnnotationForm

else:
    @final
    class ForwardRef(_Final):
        __slots__ = (
            "__forward_arg__",
            "__forward_code__",
            "__forward_evaluated__",
            "__forward_value__",
            "__forward_is_argument__",
            "__forward_is_class__",
            "__forward_module__",
        )
        __forward_arg__: str
        __forward_code__: CodeType
        __forward_evaluated__: bool
        __forward_value__: Any | None  # AnnotationForm
        __forward_is_argument__: bool
        __forward_is_class__: bool
        __forward_module__: Any | None

        def __init__(self, arg: str, is_argument: bool = True, module: Any | None = None, *, is_class: bool = False) -> None: ...

        if sys.version_info >= (3, 13):
            @overload
            @deprecated(
                "Failing to pass a value to the 'type_params' parameter of ForwardRef._evaluate() is deprecated, "
                "as it leads to incorrect behaviour when evaluating a stringified annotation "
                "that references a PEP 695 type parameter. It will be disallowed in Python 3.15."
            )
            def _evaluate(
                self, globalns: dict[str, Any] | None, localns: Mapping[str, Any] | None, *, recursive_guard: frozenset[str]
            ) -> Any | None: ...  # AnnotationForm
            @overload
            def _evaluate(
                self,
                globalns: dict[str, Any] | None,
                localns: Mapping[str, Any] | None,
                type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...],
                *,
                recursive_guard: frozenset[str],
            ) -> Any | None: ...  # AnnotationForm
        elif sys.version_info >= (3, 12):
            def _evaluate(
                self,
                globalns: dict[str, Any] | None,
                localns: Mapping[str, Any] | None,
                type_params: tuple[TypeVar | ParamSpec | TypeVarTuple, ...] | None = None,
                *,
                recursive_guard: frozenset[str],
            ) -> Any | None: ...  # AnnotationForm
        else:
            def _evaluate(
                self, globalns: dict[str, Any] | None, localns: Mapping[str, Any] | None, recursive_guard: frozenset[str]
            ) -> Any | None: ...  # AnnotationForm

        def __eq__(self, other: object) -> bool: ...
        def __hash__(self) -> int: ...
        if sys.version_info >= (3, 11):
            def __or__(self, other: Any) -> _SpecialForm: ...
            def __ror__(self, other: Any) -> _SpecialForm: ...

if sys.version_info >= (3, 10):
    def is_typeddict(tp: object) -> bool: ...

def _type_repr(obj: object) -> str: ...

if sys.version_info >= (3, 12):
    _TypeParameter: typing_extensions.TypeAlias = (
        TypeVar
        | typing_extensions.TypeVar
        | ParamSpec
        | typing_extensions.ParamSpec
        | TypeVarTuple
        | typing_extensions.TypeVarTuple
    )

    def override(method: _F, /) -> _F: ...
    @final
    class TypeAliasType:
        def __new__(cls, name: str, value: Any, *, type_params: tuple[_TypeParameter, ...] = ()) -> Self: ...
        @property
        def __value__(self) -> Any: ...  # AnnotationForm
        @property
        def __type_params__(self) -> tuple[_TypeParameter, ...]: ...
        @property
        def __parameters__(self) -> tuple[Any, ...]: ...  # AnnotationForm
        @property
        def __name__(self) -> str: ...
        # It's writable on types, but not on instances of TypeAliasType.
        @property
        def __module__(self) -> str | None: ...  # type: ignore[override]
        def __getitem__(self, parameters: Any, /) -> GenericAlias:
            """Return self[key]."""
            ...
        def __or__(self, right: Any, /) -> _SpecialForm:
            """Return self|value."""
            ...
        def __ror__(self, left: Any, /) -> _SpecialForm:
            """Return value|self."""
            ...
        if sys.version_info >= (3, 14):
            @property
            def evaluate_value(self) -> EvaluateFunc: ...

if sys.version_info >= (3, 13):
    def is_protocol(tp: type, /) -> bool: ...
    def get_protocol_members(tp: type, /) -> frozenset[str]: ...
    @final
    @type_check_only
    class _NoDefaultType: ...

    NoDefault: _NoDefaultType
    TypeIs: _SpecialForm
    ReadOnly: _SpecialForm



from typing import Any, Callable, Protocol

class C:
    def __init__(self, x):
        self.x = x

    @classmethod
    def m(cls):
        return cls(1)

def f(x, y: int):
    def g(x):
        return x * y
    z = 2 + x
    return g(z)

lambda a, b: a + b

# Use a callable Protocol to test type-based semantic token fallback when there are
# no declarations
class Run(Protocol):
    def __call__(self, *, value: str, cb: Callable[[int], int], ty: type[int], **kwargs: Any) -> None: ...

def g(run: Run) -> None:
    run(value="a", cb=lambda i: i + 1, ty=int, any=1)



from dataclasses import dataclass
from typing import TypedDict

class A:
    def __init__(self) -> None:
        pass
class B:
    def __init__(self) -> None:
        pass

class C:
    ...

class E(B):
    ...

class F(A, B): ... # error
class G(A, E): ... # error
class H(A, C): ... # no error
class I(B, C): ... # no error

@dataclass
class J:
    ...

@dataclass
class K:
    ...

@dataclass
class L:
    def __init__(self) -> None:
        pass

class M(J, K): ... # no error
class N(J, L): ... # error

class O(TypedDict):
    ...

class P(TypedDict):
    ...

class Q(O, P): ... # no error

class R:
    ...
class S:
    def __init__(self, a: int):
        super().__init__()
class T(R, S): # no error
    pass

@dataclass
class U(A, C): # error
    ...

@dataclass(init=False)
class V(A, C): # no error
    ...

class Up:
    def __init__(self): ...


class Left(Up): ...


class Right(Up): ...


class Down(Left, Right): ...  # no error


# This sample tests the handling of fields within a dataclass that
# are descriptors.

from dataclasses import dataclass

from typing import overload, Any, TypeVar, Generic, Optional, Union, Callable, Type
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    dataclass_transform,
)


_T = TypeVar("_T")


class A(Generic[_T]): ...


class Desc(Generic[_T]):
    @overload
    def __get__(self, instance: None, owner: Any) -> A[_T]: ...

    @overload
    def __get__(self, instance: object, owner: Any) -> _T: ...

    def __get__(self, instance: Optional[object], owner: Any) -> Union[A[_T], _T]: ...


@dataclass_transform(field_specifiers=(Desc[Any],))
def dataclass_like(
    *,
    init: bool = True,
    repr: bool = True,  # noqa: A002
    eq: bool = True,
    order: bool = False,
    unsafe_hash: bool = False,
) -> Callable[[Type[_T]], Type[_T]]: ...


@dataclass_like()
class B:
    x: Desc[int]
    y: Desc[str]
    z: Desc[str] = Desc()


@dataclass
class C:
    x: Desc[int]
    y: Desc[str]
    z: Desc[str] = Desc()


reveal_type(B.x, expected_text="A[int]")
reveal_type(B.y, expected_text="A[str]")
reveal_type(B.z, expected_text="A[str]")
reveal_type(C.x, expected_text="A[int]")
reveal_type(C.y, expected_text="A[str]")
reveal_type(C.z, expected_text="A[str]")

b = B(Desc(), Desc(), Desc())
reveal_type(b.x, expected_text="int")
reveal_type(b.y, expected_text="str")
reveal_type(b.z, expected_text="str")

c = C(Desc(), Desc(), Desc())
reveal_type(c.x, expected_text="int")
reveal_type(c.y, expected_text="str")
reveal_type(c.z, expected_text="str")



# This sample tests that the type analyzer flags as an error
# an attempt to assign to or delete a generic type.

from typing import Dict

# This should generate an error because assignment
# of generic types isn't allowed.
Dict[str, int] = {}

# This should generate an error because deletion
# of generic types isn't allowed.
del Dict[str, int]



# This sample validates that list comprehensions within a class
# body do not reference class-scoped variables within the
# comprehension unless they are within the initial iterator expression.

outer_var = [1, 2]


class A:
    var1 = [1, 2]
    var2 = {x for x in var1}

    # This should generate an error.
    var3 = {var1[0] for x in var1}

    var4 = {outer_var[0] for x in outer_var}



# This sample tests the detection of deprecated classes from the typing
# module.

from typing import (
    ChainMap as CM1,
    Counter as CT1,
    DefaultDict,
    Deque,
    Dict,
    FrozenSet,
    List,
    Optional,
    OrderedDict as OD1,
    Set,
    Tuple,
    Type,
    Union,
    Awaitable,
    Coroutine,
    AsyncIterable,
    AsyncGenerator,
    Iterable,
    Iterator,
    Generator,
    Reversible,
    Container,
    Collection as C1,
    Callable, # should not report an error because Callable is completely unrelated to collections
    AbstractSet,
    MutableSet,
    Mapping,
    MutableMapping,
    Sequence,
    MutableSequence,
    ByteString as BS1,
    MappingView,
    KeysView,
    ItemsView,
    ValuesView,
    ContextManager as CM1,
    AsyncContextManager,
    Pattern as P1,
    Match as M1,
)

from collections.abc import Collection, ByteString, Set as AS
from contextlib import AbstractContextManager
from re import Pattern, Match

# These should be marked deprecated for Python >= 3.9
v1: List[int] = [1, 2, 3]
v2: Dict[int, str] = {}
v3: Set[int] = set()
v4: Tuple[int] = (3,)
v5: FrozenSet[int] = frozenset()
v6: Type[int] = int
v7 = Deque()
v8 = DefaultDict()

# These should be marked deprecated for Python >= 3.10
v20: Union[int, str]
v21: Optional[int]



# This sample tests the case where a variable in an outer scope is captured
# by inner scopes and is potentially unbound.


def func1():
    if 1 + 1 > 3:
        y = 0

    class A:
        def method1(self):
            # This should generate a "possibly unbound" error.
            print(y)

            def inner():
                # This should generate a "possibly unbound" error.
                print(y)

            # This should generate a "possibly unbound" error.
            v = lambda: y

            # This should generate a "possibly unbound" error.
            x = [m + y for m in range(3)]

    def func1(self):
        # This should generate a "possibly unbound" error.
        print(y)

        def inner():
            # This should generate a "possibly unbound" error.
            print(y)

        # This should generate a "possibly unbound" error.
        v = lambda: y

        # This should generate a "possibly unbound" error.
        x = [m + y for m in range(3)]

    # The code below should not generate any errors because
    # z is assigned a value later.
    if 1 + 1 > 3:
        z = 0

    class B:
        def method1(self):
            print(z)

            def inner():
                print(z)

            v = lambda: z

            x = [m + z for m in range(3)]

    def func2(self):
        print(z)

        def inner():
            print(z)

        v = lambda: z

        x = [m + z for m in range(3)]

    z = 0



# This sample tests the case where a __call__ is marked deprecated.

from typing import Callable, Generic, ParamSpec, TypeVar
from typing_extensions import deprecated  # pyright: ignore[reportMissingModuleSource]


class A:
    @deprecated("Use ClassB instead")
    def __call__(self) -> None: ...


a = A()

# This should generate an error if reportDeprecated is enabled.
a()

P = ParamSpec("P")
R = TypeVar("R")


class B(Generic[P, R]):
    def __init__(self, cb: Callable[P, R]) -> None:
        self.cb = cb

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R:
        return self.cb(*args, **kwargs)


@B
@deprecated("Don't use this.")
def func1(x: int) -> None:
    pass


# This should generate an error if reportDeprecated is enabled.
func1(3)


def deco1(cb: Callable[P, R]) -> B[P, R]:
    return B(cb)


@deco1
@deprecated("Don't use this.")
def func2(x: int) -> None:
    pass


# This should generate an error if reportDeprecated is enabled.
func2(3)



# This sample tests type evaluation for a nested loop that involves
# accesses to an instance variable accessed through a member access
# expression that requires narrowing.

from concurrent import futures
from concurrent.futures import Future
from typing import Any, Dict, Optional


class A:
    def __init__(self):
        self.pending: Optional[Dict[Future[Any], int]]
        self.foo: bool

    def poll(self):
        assert self.pending is not None
        while True:
            if self.pending:
                pass

            ready, _ = futures.wait(self.pending)

            for future_id in ready:
                self.pending.pop(future_id)

                future_id.result()
                if self.foo:
                    pass



# This sample tests the detection of keywords in a named tuple
# definition and support for the "rename" parameter.


from collections import namedtuple
from typing import NamedTuple


# This should generate an error because "def" is a keyword.
NT1 = namedtuple("NT1", ["abc", "def"])

# This should generate an error because "class" is a keyword.
NT2 = namedtuple("NT2", ["abc", "class"], rename=False)

NT3 = namedtuple("NT3", ["abc", "def"], rename=True)

v3 = NT3(abc=0, _1=0)

# This should generate an error because "def" is a keyword.
NT4 = NamedTuple("NT4", [("abc", int), ("def", int)])


# These are soft keywords, so they shouldn't generate an error.
NT5 = namedtuple("NT5", ["type", "match"])



# This sample verifies that functions are treated as though they
# derive from object.

from typing import Callable, overload


@overload
def func1(a: str) -> str: ...


@overload
def func1(a: int) -> int: ...


def func1(a: str | int) -> str | int: ...


def func2(a: str | int) -> str | int: ...


def takes_object(val: object) -> None: ...


takes_object(func1)
takes_object(func2)


def func3(b: Callable[[str], bool]) -> None:
    if b == func1:
        pass

    if b != func2:
        pass



# This sample tests that final variables imported from another module
# cannot be overwritten.

from .final7 import *

# This should generate an error.
var1 = 1

# This should generate an error.
var2 = 1


def func1():
    from .final7 import var1, var2

    # This should generate an error.
    var1 = 1

    # This should generate an error.
    var2 = 1



# This sample tests the handling of an unpacked TypedDict passed as
# an argument to a function.

from typing import TypedDict, Unpack


class TD1(TypedDict):
    arg1: int
    arg2: str


class TD2(TD1):
    arg3: float


def func1(arg1: int, arg2: str):
    pass


def func2(arg1: int, arg2: str, arg3: float):
    pass


def func3(arg1: int, arg2: str, **kwargs: float):
    pass


def func4(arg1: int, arg2: str, **kwargs: int):
    pass


def func5(arg1: int, arg2: str, **kwargs: object):
    pass


td1: TD1 = {"arg1": 10, "arg2": "something"}
td2: TD2 = {"arg1": 10, "arg2": "something", "arg3": 3.4}

func1(**td1)

# This should generate an error because "arg1" is already assigned
func1(arg1=3, **td1)

# This should generate an error because "arg3" isn't provided
func1(**td2)

# This should generate an error because "arg3" isn't matched
func2(**td1)

func2(**td2)


# This should generate an error because the extra entries
# in the TD are of type object.
func3(**td1)

# This should generate an error because the extra entries
# in the TD are of type object.
func3(**td2)

# This should generate an error because the extra entries
# in the TD are of type object.
func4(**td1)

func5(**td1)
func5(**td2)

# This should generate two errors because "arg3" cannot be matched
# due to the type of the **kwargs parameter. Also, the extra entries
# in the TD are of type object.
func4(**td2)


class Options(TypedDict, total=False):
    opt1: bool
    opt2: str


def func6(code: str | None = None, **options: Unpack[Options]):
    pass


func6(**{})
func6(**{"opt1": True})
func6(**{"opt2": "hi"})



# This sample tests a case where a generic function is passed as
# an argument to itself, creating a recursive situation that
# caused an infinite loop.

from random import random
from typing import Any, Callable, TypeVar

T = TypeVar("T")
U = TypeVar("U")


def func1(x: T, y: U) -> T | U:
    return x if random() > 0.5 else y


def func2(x: T, y: T) -> T:
    return x if random() > 0.5 else y


reveal_type(
    func2(func1, func2), expected_text="(x: T(1)@func2, y: T(1)@func2) -> T(1)@func2"
)


S = TypeVar("S", bound=Callable[..., Any])


def func3(x: S) -> S:
    return x


reveal_type(func3(func3), expected_text="(x: S(1)@func3) -> S(1)@func3")



# This sample tests the handling of recursive type aliases that are generic.

from __future__ import annotations
from typing import Mapping, Sequence, TypeVar, Union

S = TypeVar("S")
RecList = Union[Mapping[str, "RecList[S]"], Sequence["RecList[S]"], S]

T3 = TypeVar("T3", RecList[int], RecList[str])


def f3(x: RecList[int] | RecList[str]) -> None: ...


def g3(x: T3):
    return f3(x)


def f4(x: RecList[str] | RecList[int]) -> None: ...


def g4(x: T3):
    return f4(x)



# This sample tests the use of @functools.cache with a property.

from functools import cache


class Rectangle:
    def __init__(self, length: int, width: int) -> None:
        self._length = length
        self._width = width

    @property
    @cache
    def area(self) -> int:
        return self._length * self._width


def is_large_rectangle(rec: Rectangle) -> bool:
    print(rec.area)
    return rec.area >= 100


rec = Rectangle(10, 10)
print(is_large_rectangle(rec))



# This sample tests the detection of deprecated classes from the
# typing_extensions module.

# This test is heavily derived from deprecatedAlias1.py.

from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    ChainMap as CM1,
    Counter as CT1,
    DefaultDict,
    Deque,
    Dict,
    FrozenSet,
    List,
    Optional,
    OrderedDict as OD1,
    Set,
    Tuple,
    Type,
    Union,
    Awaitable,
    Coroutine,
    AsyncIterable,
    AsyncGenerator,
    Iterable,
    Iterator,
    Generator,
    Reversible,
    Container,
    Collection as C1,
    Callable,
    AbstractSet,
    MutableSet,
    Mapping,
    MutableMapping,
    Sequence,
    MutableSequence,
    # ByteString as BS1, # This symbol doesn't exist in typing_extensions
    MappingView,
    KeysView,
    ItemsView,
    ValuesView,
    ContextManager as CM1,  # This symbol is reexported from contextlib
    AsyncContextManager,  # This symbol is reexported from contextlib
    Pattern as P1,
    Match as M1,
)

from collections.abc import Collection, ByteString, Set as AS
from contextlib import AbstractContextManager
from re import Pattern, Match

# These should be marked deprecated for Python >= 3.9
v1: List[int] = [1, 2, 3]
v2: Dict[int, str] = {}
v3: Set[int] = set()
v4: Tuple[int] = (3,)
v5: FrozenSet[int] = frozenset()
v6: Type[int] = int
v7 = Deque()
v8 = DefaultDict()

# These should be marked deprecated for Python >= 3.10
v20: Union[int, str]
v21: Optional[int]



# This sample tests the case where an issubclass type guard narrows
# to an abstract base class. When attempting to instantiate the
# class, there should be no "cannot instantiate ABC" error.

# pyright: strict

from abc import ABC, abstractmethod
from typing import Any


class Base(ABC):
    @abstractmethod
    def f(self) -> None: ...


def func1(cls: Any):
    assert issubclass(cls, Base)
    reveal_type(cls, expected_text="type[Base]")
    _ = cls()


def func2(cls: Any):
    assert isinstance(cls, type)
    reveal_type(cls, expected_text="type")
    assert issubclass(cls, Base)
    reveal_type(cls, expected_text="type[Base]")
    _ = cls()



# This test validates that NamedTuple instances can be destructured
# and indexed to get to their constituent element types.

from typing import NamedTuple


class MyDataClass(NamedTuple):
    entry_1: str
    entry_2: int


nt1 = MyDataClass("yes", 1)

(a1, a2) = nt1
a1_1: str = a1
a2_1: int = a2

# These should generate an error because a1 and a2 are
# the wrong types.
a1_2: int = a1
a2_2: str = a2


b1 = nt1[0]
b2 = nt1[1]
b1_1: str = b1
b2_1: int = b2

# These should generate an error because a1 and a2 are
# the wrong types.
b1_2: int = b1
b2_2: str = b2


MyNT = NamedTuple("MyNT", [("hi", int), ("bye", str)])

nt2 = MyNT(3, "yo")

(c1, c2) = nt2
c1_2: int = c1
c2_2: str = c2

# These should generate an error because a1 and a2 are
# the wrong types.
c1_1: str = c1
c2_1: int = c2

d1 = nt2[0]
d2 = nt2[1]
d1_2: int = d1
d2_2: str = d2

# These should generate an error because a1 and a2 are
# the wrong types.
d1_1: str = d1
d2_1: int = d2



# This sample tests the handling of `Union[*Ts]` in certain cases.

# Enable experimental features to support Union[*Ts].
# pyright: enableExperimentalFeatures=true

from typing import Generic, TypeVarTuple, Union

Ts = TypeVarTuple("Ts")


class ClassA(Generic[*Ts]):
    def __init__(self) -> None:
        self.x: list[Union[*Ts]] = []

        reveal_type(self.x, expected_text="list[Union[*Ts@ClassA]]")

    def method(self) -> Union[*Ts]: ...


a1 = ClassA[int, bool, str]()

reveal_type(a1.method(), expected_text="int | bool | str")
reveal_type(a1.x, expected_text="list[int | bool | str]")


def func1(t0: tuple[*Ts], t1: tuple[*Ts]):
    return all(v0 == v1 for v0, v1 in zip(t0, t1))



# This sample tests the provision in PEP 544 where a class type can
# be assigned to a protocol.

from typing import Any, ClassVar, Protocol


class ProtoA(Protocol):
    def meth(_self, x: int) -> int: ...


class ProtoB(Protocol):
    def meth(_self, self: Any, x: int) -> int: ...


class C:
    def meth(self, x: int) -> int: ...


# This should generate an error because C.meth isn't compatible
# with ProtoA().meth.
a: ProtoA = C

b: ProtoB = C


class ProtoD(Protocol):
    var1: int

    @property
    def var2(self) -> str: ...


class E:
    var1: ClassVar[int]
    var2: ClassVar[str]


class F:
    var1: ClassVar[int]
    var2: ClassVar[int]


d: ProtoD = E

# This should generate an error because var2 is the wrong type.
e: ProtoD = F


class Jumps(Protocol):
    def jump(self) -> int: ...


class Jumper1:
    @classmethod
    def jump(cls) -> int: ...


class Jumper2:
    def jump(self) -> int: ...


def do_jump(j: Jumps):
    print(j.jump())


do_jump(Jumper1)
do_jump(Jumper2())


class ProtoG1(Protocol):
    attr1: ClassVar[int]


class ProtoG2(Protocol):
    attr1: int


class ConcreteG1:
    attr1: ClassVar[int] = 1


class ConcreteG2:
    attr1: int = 1


class GMeta(type):
    attr1: int


class ConcreteG3(metaclass=GMeta):
    pass


# The following four lines should generate an error.
pc1: ProtoG1 = ConcreteG1
pc2: ProtoG1 = ConcreteG2
pc3: ProtoG2 = ConcreteG2
pc4: ProtoG1 = ConcreteG3

pc5: ProtoG2 = ConcreteG1
pc6: ProtoG2 = ConcreteG3



# This sample tests the handling of overloads with a ParamSpec.

from typing import Callable, Concatenate, overload, TypeVar, ParamSpec

P = ParamSpec("P")
R = TypeVar("R")


def callable1(
    func: Callable[P, R], *args: P.args, **kwargs: P.kwargs
) -> Callable[[], R]: ...


@overload
def func1() -> None: ...


@overload
def func1(a: int) -> None: ...


def func1(a: int = 1) -> None: ...


callable1(func1)
callable1(func1, 1)
callable1(func1, a=1)

# This should generate an error because none of the overloads
# captured by the ParamSpec match those arguments.
callable1(func1, 1, 2)

# This should generate an error because none of the overloads
# captured by the ParamSpec match those arguments.
callable1(func1, b=2)


def callable2(
    func: Callable[Concatenate[int, P], R], *args: P.args, **kwargs: P.kwargs
) -> Callable[[], R]: ...


@overload
def func2() -> None: ...


@overload
def func2(a: int) -> int: ...


@overload
def func2(a: int, b: str) -> str: ...


def func2(a: int = 1, b: str = "") -> None | int | str: ...


callable2(func2)
callable2(func2, "")
callable2(func2, b="")

# This should generate an error because none of the overloads
# captured by the ParamSpec match those arguments.
callable2(func2, 1, "")


def callable3(func: Callable[P, R]) -> Callable[Concatenate[int, P], R]: ...


c3_2 = callable3(func2)
c3_2(1)
c3_2(1, a=1)
c3_2(1, 1, b="")

# This should generate an error because none of the overloads
# match these arguments.
c3_2(1, "")

# This should generate an error because none of the overloads
# match these arguments.
c3_2(1, 1, c="")


@overload
def func3(x: int) -> None: ...


@overload
def func3(x: str) -> None: ...


def func3(x) -> None:
    pass


def callable4(func: Callable[P, R], *args: P.args, **kwargs: P.kwargs) -> R: ...


callable4(func3, 1)
callable4(func3, x=1)
callable4(func3, "")
callable4(func3, x="")

# This should generate an error.
callable4(func3, 1.0)

# This should generate two errors because x is missing and y is unknown.
callable4(func3, y=1)


@overload
def func4(x: str) -> str: ...


@overload
def func4(x: int) -> int: ...


def func4(x: str | int):
    return x


def callable5(f: Callable[P, R]):
    def inner(*args: P.args, **kwargs: P.kwargs) -> list[R]:
        return [f(*args, **kwargs)]

    return inner


callable5(func4)(0)
callable5(func4)("")



# This sample tests the case where an unpacked tuple argument in a call
# expression is matched to an `*args` parameter that has a declared type
# that includes an unpacked TypeVarTuple.

from typing_extensions import TypeVarTuple  # pyright: ignore[reportMissingModuleSource]

Ts = TypeVarTuple("Ts")


def call0(*args: *Ts) -> tuple[*Ts]: ...


def call1(*args: *tuple[int, *Ts]) -> tuple[*Ts]: ...


def call2(*args: *tuple[*Ts, float]) -> tuple[*Ts]: ...


def call3(*args: *tuple[int, *Ts, float]) -> tuple[*Ts]: ...


def call4(*args: *tuple[*tuple[int, *tuple[*Ts], float]]) -> tuple[*Ts]: ...


def func1(*args: *tuple[int, str]):
    reveal_type(call0(*args), expected_text="tuple[int, str]")


def func2(*args: *tuple[int, ...]):
    reveal_type(call0(*args), expected_text="tuple[int, ...]")


def func3(*args: *tuple[int, *tuple[str, ...], float]):
    reveal_type(call0(*args), expected_text="tuple[int, *tuple[str, ...], float]")


def func4(*args: *Ts) -> tuple[*Ts]:
    call0(*args)
    return args


def func5(x: int, y: str, z: float):
    v1 = call1(*(x, y, z))
    reveal_type(v1, expected_text="tuple[str, float]")

    v2 = call2(*(x, y, z))
    reveal_type(v2, expected_text="tuple[int, str]")

    v3 = call3(*(x, y, z))
    reveal_type(v3, expected_text="tuple[str]")

    v4 = call4(*(x, *(y, z)))
    reveal_type(v4, expected_text="tuple[str]")


def func6(*args: *tuple[int, *tuple[None, ...], float]):
    reveal_type(call2(*args), expected_text="tuple[int, *tuple[None, ...]]")



# This sample tests the handling of multiple unpack operators in a
# star expression.

a = [1, 2]
b = ["3", "4"]

# This should generate an error for versions of Python <3.9
for x in *a, *b:
    print(x)

c = *a, *b
print(c)

# This should always generate an error.
*a, *b = (1, 2)


def func1(x: str):
    "".join([*sorted([x])])



# This sample tests that wildcard imports are not allowed
# outside of the module scope.

from .import5 import *


class A:
    # This should generate an error.
    from .import5 import *


def func1():
    # This should generate an error.
    from .import5 import *



# This sample tests the TypeIs form.

# pyright: reportMissingModuleSource=false

from typing import (
    Any,
    Callable,
    Collection,
    Literal,
    Mapping,
    Sequence,
    TypeVar,
    Union,
    overload,
)

from typing_extensions import TypeIs


def is_str1(val: Union[str, int]) -> TypeIs[str]:
    return isinstance(val, str)


def func1(val: Union[str, int]):
    if is_str1(val):
        reveal_type(val, expected_text="str")
    else:
        reveal_type(val, expected_text="int")


def is_true(o: object) -> TypeIs[Literal[True]]: ...


def func2(val: bool):
    if not is_true(val):
        reveal_type(val, expected_text="bool")
    else:
        reveal_type(val, expected_text="Literal[True]")

    reveal_type(val, expected_text="bool")


def is_list(val: object) -> TypeIs[list[Any]]:
    return isinstance(val, list)


def func3(val: dict[str, str] | list[str] | list[int] | Sequence[int]):
    if is_list(val):
        reveal_type(val, expected_text="list[str] | list[int] | list[Any]")
    else:
        reveal_type(val, expected_text="dict[str, str] | Sequence[int]")


def func4(val: dict[str, str] | list[str] | list[int] | tuple[int]):
    if is_list(val):
        reveal_type(val, expected_text="list[str] | list[int]")
    else:
        reveal_type(val, expected_text="dict[str, str] | tuple[int]")


_K = TypeVar("_K")
_V = TypeVar("_V")


def is_dict(val: Mapping[_K, _V]) -> TypeIs[dict[_K, _V]]:
    return isinstance(val, dict)


def func5(val: dict[_K, _V] | Mapping[_K, _V]):
    if not is_dict(val):
        reveal_type(val, expected_text="Mapping[_K@func5, _V@func5]")
    else:
        reveal_type(val, expected_text="dict[_K@func5, _V@func5]")


def is_cardinal_direction(val: str) -> TypeIs[Literal["N", "S", "E", "W"]]:
    return val in ("N", "S", "E", "W")


def func6(direction: Literal["NW", "E"]):
    if is_cardinal_direction(direction):
        reveal_type(direction, expected_text="Literal['E']")
    else:
        reveal_type(direction, expected_text="Literal['NW']")


class Animal: ...


class Kangaroo(Animal): ...


class Koala(Animal): ...


T = TypeVar("T")


def is_marsupial(val: Animal) -> TypeIs[Kangaroo | Koala]:
    return isinstance(val, Kangaroo | Koala)


# This should generate an error because list[T] isn't consistent with list[T | None].
def has_no_nones(val: list[T | None]) -> TypeIs[list[T]]:
    return None not in val


def takes_int_typeis(f: Callable[[object], TypeIs[int]]) -> None:
    pass


def int_typeis(val: object) -> TypeIs[int]:
    return isinstance(val, int)


def bool_typeis(val: object) -> TypeIs[bool]:
    return isinstance(val, bool)


takes_int_typeis(int_typeis)

# This should generate an error because TypeIs is invariant.
takes_int_typeis(bool_typeis)


def is_two_element_tuple(val: tuple[T, ...]) -> TypeIs[tuple[T, T]]:
    return len(val) == 2


def func7(names: tuple[str, ...]):
    if is_two_element_tuple(names):
        reveal_type(names, expected_text="tuple[str, str]")
    else:
        reveal_type(names, expected_text="tuple[str, ...]")


def is_int(obj: type) -> TypeIs[type[int]]: ...


def func8(x: type) -> None:
    if is_int(x):
        reveal_type(x, expected_text="type[int]")


def is_int_list(x: Collection[Any]) -> TypeIs[list[int]]:
    raise NotImplementedError


def func9(val: Collection[object]) -> None:
    if is_int_list(val):
        reveal_type(val, expected_text="list[int]")
    else:
        reveal_type(val, expected_text="Collection[object]")


@overload
def func10(v: tuple[int | str, ...], b: Literal[False]) -> TypeIs[tuple[str, ...]]: ...


@overload
def func10(
    v: tuple[int | str, ...], b: Literal[True] = True
) -> TypeIs[tuple[int, ...]]: ...


def func10(v: tuple[int | str, ...], b: bool = True) -> bool: ...


v0 = is_int(int)
v1: bool = v0
v2: int = v0
v3 = v0 & v0


def is_sequence_of_int(sequence: Sequence) -> TypeIs[Sequence[int]]:
    return all(isinstance(x, int) for x in sequence)


def func11(v: Sequence[int] | Sequence[str]):
    if is_sequence_of_int(v):
        reveal_type(v, expected_text="Sequence[int]")
    else:
        reveal_type(v, expected_text="Sequence[str]")


def func12(v: Sequence[int | str] | Sequence[list[Any]]):
    if is_sequence_of_int(v):
        reveal_type(v, expected_text="Sequence[int]")
    else:
        reveal_type(v, expected_text="Sequence[int | str] | Sequence[list[Any]]")



from _typeshed import Incomplete

class EndOfText(RuntimeError): ...

class Scanner:
    data: Incomplete
    data_length: Incomplete
    start_pos: int
    pos: int
    flags: Incomplete
    last: Incomplete
    match: Incomplete
    def __init__(self, text, flags: int = 0) -> None: ...
    @property
    def eos(self): ...
    def check(self, pattern): ...
    def test(self, pattern): ...
    def scan(self, pattern): ...
    def get_char(self) -> None: ...



# This sample tests the reportMissingSuperCall diagnostic check.
from abc import abstractmethod
from typing import final


class ParentA:
    pass


class ParentB:
    # This should generate an error because it's missing a super().__init__ call.
    def __init__(self):
        pass


class ParentBPrime(ParentB):
    pass


class ParentC:
    pass


@final
class ParentD:
    def __init__(self):
        pass

    def __init_subclass__(cls) -> None:
        pass


class ChildA(ParentA, ParentB):
    # This should generate an error.
    def __init__(self):
        pass

    # This should generate an error.
    def __init_subclass__(cls) -> None:
        pass


class ChildB(ParentA, ParentB):
    def __init__(self):
        super().__init__()


class ChildC1(ParentA, ParentB):
    def __init__(self):
        ParentB.__init__(self)


class ChildC2(ParentA, ParentB):
    def __init__(self):
        ParentA.__init__(self)
        ParentB.__init__(self)


class ChildCPrime(ParentA, ParentBPrime, ParentC):
    def __init__(self):
        super(ParentBPrime).__init__()


class ChildD(ParentC):
    # This should generate an error.
    def __init__(self):
        pass


@final
class ChildE(ParentC):
    def __init__(self):
        pass

class BaseClassWithAbstractMethod:
    @abstractmethod
    def __enter__(self) -> None: ...

class DerivedClassThatDoesntCassSuper1(BaseClassWithAbstractMethod):
    def __enter__(self) -> None: # no error
        ...

class BaseClassWithNormalMethod:
    def __enter__(self) -> None: ...

class DerivedClassThatDoesntCassSuper2(BaseClassWithNormalMethod):
    def __enter__(self) -> None: # error
        ...



# This sample tests that type declarations for class members do not
# include method-local type variables.

from typing import TypeVar

T1 = TypeVar("T1")
T2 = TypeVar("T2")


def func(x: list[T1]):
    class ClassB:
        def __init__(self, val: list[T2]):
            # This should generate an error because T is scoped to
            # the method.
            self.a1: list[T2] = val

            self.a2: list[T1] = []

        @classmethod
        def method_b(cls, val: list[T2]):
            # This should generate an error because T is scoped to
            # the method.
            cls.b1: list[T2] = val

            cls.b2: list[T1] = []

        def method_c(self, val: list[T2]):
            # This should generate an error because T is scoped to
            # the method.
            self.c1: list[T2] = val

            self.c2: list[T1] = []

    b = ClassB([])
    reveal_type(b.a1, expected_text="list[Unknown]")
    reveal_type(b.b1, expected_text="list[Unknown]")
    reveal_type(b.c1, expected_text="list[Unknown]")

    return b



# This sample tests the handling of default parameter value
# expressions in a lambda.


from typing import Callable, Protocol


def test1():
    var = 1

    lambda _=var: ...


def test2():
    # This should generate an error because var2 isn't defined.
    lambda _=var2: ...


def test3():
    var = 0
    lambda var=var: ...


class MyCallback(Protocol):
    def __call__(self, y: int, a: int = 0) -> bool: ...


lambda1: Callable[[int, int], bool] = lambda y, a=0: a == y
lambda2: MyCallback = lambda y, a=0: a == y

lambda1(20)
lambda2(20)
lambda2(20, 30)



# This sample tests member access "set" operations when the target
# member is an object that doesn't provide a __set__ method.

# pyright: strict

from typing import Any, Generic, TypeVar, overload


_T = TypeVar("_T")


class ParentA:
    pass


class Column(Generic[_T]):
    def __init__(self, type: type[_T]) -> None: ...

    @overload
    def __get__(self: "Column[_T]", instance: None, type: Any) -> "Column[_T]": ...

    @overload
    def __get__(self: "Column[_T]", instance: ParentA, type: Any) -> _T: ...

    def __get__(
        self, instance: ParentA | None, type: Any
    ) -> _T | None | "Column[_T]": ...


class ChildA(ParentA):
    attr1: Column[str] = Column(str)
    attr2 = Column(str)


ChildA.attr1
ChildA().attr1
ChildA.attr2
ChildA().attr2

foo = ChildA()

# This should generate an error because bar is declared as containing a
# Column[str], which doesn't provide a __set__ method.
foo.attr1 = ""

# This should generate an error because baz's inferred type is
# Column[str], which doesn't provide a __set__ method.
foo.attr2 = ""



# This sample verifies that bitwise or operator is not
# interpreted as a Union operator in cases where it
# shouldn't be.

from typing import Any


class Class1:
    def __init__(self, v):
        self._v = v


def test_bad_syntax(a: Any, b: Any):
    r = Class1(a | b)
    print(r)



# This sample exercises the type analyzer's type narrowing
# logic for tests of the form "type(X) == Y" or "type(X) != Y".

from typing import Any, Generic, TypeVar, final


def func1(a: str | int) -> int:
    if type(a) != str:
        # This should generate an error because
        # "a" is potentially a subclass of str.
        return a

    # This should generate an error because
    # "a" is provably type str at this point.
    return a


def func2(a: str | None) -> str:
    if type(a) == str:
        return a

    # This should generate an error because
    # "a" is provably type str at this point.
    return a


def func3(a: dict[str, Any]) -> str:
    val = a.get("hello")
    if type(val) == str:
        return val

    return "none"


class A:
    pass


class B(A):
    pass


def func4(a: str | A):
    if type(a) == B:
        reveal_type(a, expected_text="B")
    else:
        reveal_type(a, expected_text="str | A")


T = TypeVar("T")


class C(Generic[T]):
    def __init__(self, a: T):
        self.a = a


class D:
    pass


E = C[T] | D


def func5(x: E[T]) -> None:
    if type(x) == C:
        reveal_type(x, expected_text="C[T@func5]")


@final
class AFinal:
    pass


@final
class BFinal:
    pass


def func6(val: AFinal | BFinal) -> None:
    if type(val) == AFinal:
        reveal_type(val, expected_text="AFinal")
    else:
        reveal_type(val, expected_text="BFinal")


def func7(val: Any):
    if type(val) == int:
        reveal_type(val, expected_text="int")
    else:
        reveal_type(val, expected_text="Any")

    reveal_type(val, expected_text="int | Any")


class CParent: ...


class CChild(CParent): ...


_TC = TypeVar("_TC", bound=CParent)


def func8(a: _TC, b: _TC) -> _TC:
    if type(a) == CChild:
        reveal_type(a, expected_text="CChild*")
        return a
    reveal_type(a, expected_text="CParent*")
    return a



# This sample tests assignment expressions used within arguments.

from dataclasses import dataclass
from typing import Mapping
import collections


class NearestKeyDict(collections.UserDict):
    def method1(self, key):
        a = len(keys := [k for k in sorted(self.data) if k >= key])

        # This should generate an error because walrus operators
        # are not allowed with named arguments.
        b = list(iterable = keys := [k for k in sorted(self.data) if k >= key])


@dataclass
class DC1:
    x: str


def func1(mapping: Mapping[str, dict]):
    return [DC1(temp := "x", **mapping[temp])]



# This sample tests the handling of PEP 695 type parameter syntax used for
# bounded and constrained TypeVars, TypeVarTuples, and ParamSpecs.

from typing import Any

class ClassA[R, *Ts, **P]:
    ...

class ClassB[R: int | str]:
    ...

# This should generate an error because 'dummy' is not declared.
class ClassC[R: dummy]:
    ...

class ClassD[R: "ClassE[Any]"]:
    ...


class ClassE[T]:
    ...

# This should generate an error because variadic type params don't 
# support bound expressions.
class ClassF[*Ts: int]: ...

# This should generate an error because ParamSpecs don't 
# support bound expressions.
class ClassG[**P: int]: ...

# This should generate an error because the expression isn't
# a valid type.
class ClassH[R: 1]: ...

# This should generate an error because a constrained type
# must contain at least two types.
class ClassI[R: ()]: ...

# This should generate an error because a constrained type
# must contain at least two types.
class ClassJ[R: (int, )]: ...

class ClassK[R: (bytes, str)]: ...

t2 = (bytes, str)
# This should generate an error because a literal tuple expression
# must be used for constrained types.
class ClassL[R: t2]: ...

# This should generate an error because constraints must be legal
# type expressions.
class ClassM[R: (1, str)]: ...

v: type[int] = int

# This should generate an error because constraints must be legal
# type expressions.
class ClassN[R: (v, str)]: ...



# This sample tests that type aliases can consist of
# partially-specialized classes that can be further
# specialized.

# pyright: reportMissingModuleSource=false

from typing import Callable, Generic, Optional
from typing_extensions import ParamSpec, TypeVar

T = TypeVar("T")
P = ParamSpec("P")
TStr = TypeVar("TStr", default=str)

ValidationResult = tuple[bool, Optional[T]]


def foo() -> ValidationResult[str]:
    return False, "valid"


class ClassA(Generic[T]):
    def __new__(cls, value: T) -> "ClassA[T]": ...


TypeAliasA1 = ClassA[T]

a1 = ClassA(3.0)
reveal_type(a1, expected_text="ClassA[float]")

a2 = TypeAliasA1(3)
reveal_type(a2, expected_text="ClassA[Unknown]")

a3 = TypeAliasA1[int](3)
reveal_type(a3, expected_text="ClassA[int]")


TypeAliasA2 = ClassA[TStr]

# This should generate an error.
b1 = TypeAliasA2(1)

b2 = TypeAliasA2("")
reveal_type(b2, expected_text="ClassA[str]")

b3 = TypeAliasA2[float](1.0)
reveal_type(b3, expected_text="ClassA[float]")

Func = Callable[P, T]
AnyFunc = Func[P, int]
x: AnyFunc[...]



# This sample tests the special-case handling of properties that return
# generics within a protocol.

from functools import partial
from typing_extensions import (  # pyright: ignore[reportMissingModuleSource]
    Protocol,
    Self,
)
from typing import Any, Callable, TypeVar

_T = TypeVar("_T", covariant=True)


class Partial(Protocol[_T]):
    @property
    def func(self) -> Callable[..., _T]: ...

    def __new__(
        cls: type[Self], __func: Callable[..., _T], *args: Any, **kwargs: Any
    ) -> Self: ...


def func1(x: Partial[int]): ...


func1(partial(int))



# This sample tests the "Final" type annotation
# introduced in Python 3.8.

import typing
from typing import Annotated, Any, Final, Protocol, TypeVar

T = TypeVar("T")

v1: typing.Final = 3

must_be_int: int = v1

# This should generate an error because
# reassignment of a Final variable should
# not be allowed.
v1 = 4

# This should generate an error because there
# is a previous Final declaration.
v1: Final[int]

# This should generate an error because
# the type doesn't match.
v2: Final[str] = 3

# This should generate an error because
# we expect only one type argument for Final.
v3: Final[str, int] = "hello"


v4: Final = 5
reveal_type(v4, expected_text="Literal[5]")


class ClassA:
    member1: Final = 4

    # This should generate an error because only
    # one declaration can have a Final attribute.
    member1: Final

    member2: typing.Final[int] = 3

    # this should generate an error because it's a redeclaration
    member4: Final[int]

    # This should generate an error because there is
    # no assignment.
    member5: Final[str]

    member6: Final[int]

    _member7: Final = 6
    __member8: Final = 6

    member9: Final = 2

    # This should generate an error.
    member9 = 3

    def __init__(self, a: bool):
        # This should generate an error because a Final
        # member outside of a stub file or a class body
        # must have an initializer.
        self.member3: Final

        # This should generate an error because this symbol
        # already has a final declaration.
        self.member2: Final[int]

        if a:
            self.member4 = 5
        else:
            self.member4 = 6

        self.member4 = 6

    def another_method(self):
        # This should generate an error because assignments
        # can occur only within class bodies or __init__ methods.
        self.member6 = 4

        # This should generate an error because 'Final' cannot
        # be used to annotate instance variables outside of
        # an __init__ method.
        self.member7: Final = 6


reveal_type(ClassA.member1, expected_text="Literal[4]")
reveal_type(ClassA(True).member1, expected_text="Literal[4]")


class ClassB(ClassA):
    # This should generate an error because we are overriding
    # a member that is marked Final in the parent class.
    member1 = 5

    # This should generate an error because we are overriding
    # a member that is marked Final in the parent class.
    _member7: Final = 6

    # This should not generate an error because it's private.
    __member8: Final = 6

    def __init__(self):
        # This should generate an error because we are overriding
        # a member that is marked Final in the parent class.
        self.member6 = 5


# This should generate an error because Final isn't allowed for
# function parameters.
def func1(a: Final[int]):
    pass


# This should generate an error because Final must the outermost
# type in assignments.
b: list[Final[int]] = []


class ClassC:
    member1: Final = 3
    member2: Final
    member4: Final
    member5: Final = 3

    def __init__(self):
        # This should generate an error.
        self.member1 = 5

        self.member2 = "hi"

        self.member3: Final = "hi"

        # This should generate an error.
        ClassC.member4 = "hi"

        # This should generate an error.
        ClassC.member5 = 3

    def other(self):
        # This should generate an error.
        self.member1 = 5

        # This should generate an error.
        self.member2 = "hi"

        # This should generate an error.
        self.member3 = "hi"


a = ClassC()

# This should generate an error.
a.member1 = 4

# This should generate an error.
a.member3 = "x"


def func2():
    x: Final[Any] = 3

    # This should generate an error because x is Final.
    x += 1

    # This should generate an error because x is Final.
    a = (x := 4)

    # This should generate an error because x is Final.
    for x in [1, 2, 3]:
        pass

    # This should generate an error because x is Final.
    with open("Hi") as x:
        pass

    try:
        pass
    # This should generate an error because x is Final.
    except ModuleNotFoundError as x:
        pass

    # This should generate an error because x is Final.
    (a, x) = (1, 2)


class ClassD:
    def __init__(self):
        self.x: Final = 1

    def method1(self):
        # This should generate an error because x is Final.
        self.x += 1


class ClassE(Protocol):
    x: Final[int]


def func3(x: type[T]) -> T:
    return x()


# This should generate an error because Final isn't compatible with type.
func3(Final[int])


v5: Final = lambda: None


# This should generate an error because foo5 is declared as Final.
def v5() -> None:
    pass


# This should generate an error because ClassVar is Final.
from typing import ClassVar

ClassVar: Final = 3


v6: Annotated[Final[int], "meta"] = 1

# This should generate an error
v6 = 2

v7: Annotated[Annotated[Final[int], "meta"], "meta"] = 1

# This should generate an error
v7 = 2

v8: Annotated[Final, "meta"] = 1

# This should generate an error
v8 = 2

v9: Final = 2 or "2"
reveal_type(v9, expected_text="Literal[2]")

v10: Final = 0 or "2"
reveal_type(v10, expected_text="Literal['2']")

v11: Final = b"" and True
reveal_type(v11, expected_text='Literal[b""]')

v12: Final = b"2" and True
reveal_type(v12, expected_text="Literal[True]")


def func4():
    while 1 < 1:
        # This should generate an error because it's in a loop.
        x1: Final = 1

    for i in range(10):
        if i < 3:
            # This should generate an error because it's in a loop.
            x2: Final[int] = 1


class ClassF:
    while 1 < 2:
        # This should generate an error because it's in a loop.
        x1: Final = 1

    for i in range(10):
        if i < 3:
            # This should generate an error because it's in a loop.
            x2: Final[int] = 1



# This sample verifies that the type checker is using
# synthesized type variables for "self" and "cls" variables.


class BaseClass:
    @classmethod
    def c(cls):
        return cls

    def f(self):
        return self


class SubClass(BaseClass):
    pass


def requires_subclass(p1: SubClass):
    pass


x = SubClass().f()
requires_subclass(x)

y = SubClass().c()
requires_subclass(y())



# This sample tests that a union that includes two types that are subtypes
# of each other (like tuple[Any] and tuple[int]) is handled correctly
# when performing type compatibility tests.

from typing import Any


def func(t: tuple[Any] | tuple[int]):
    # This should generate a type violation.
    x: int = t



# This sample tests handling of type aliases.

from datetime import datetime
from typing import Any, Callable, Generic, TypeVar, Union


from typing import TypeVar, Union, Optional

S = TypeVar("S")

Response1 = Optional[Union[S, int]]


def f1_1() -> Response1[str]:
    return None


def f1_2() -> Response1[str]:
    return "s"


def f1_3() -> Response1[float]:
    # This should generate an error.
    return "s"


Response2 = Union[S, int]


def f2_1() -> Response2[Any]:
    return "s"


def f2_2() -> Response2[str]:
    return "s"


def f2_3() -> Response2[float]:
    return 3.4


def f2_4() -> Response2[datetime]:
    # This should generate an error
    return 3.4


Response3 = Callable[[S], S]


def response2(query: str) -> Response3[int]:
    return lambda x: x + 2


def response3(query: str) -> Response3[datetime]:
    # This should generate an error because datetime doesn't support +
    return lambda x: x + 2


Response4 = Union[S, int, str]


class Foo1:
    pass


class Foo2:
    pass


class Foo3:
    pass


T = TypeVar("T")

Response5 = Union[T, Foo1, Foo2]

# Test nested type aliases
Response6 = Response5[Response4[Foo3]]


def f6_1() -> Response6:
    return Foo1()


def f6_2() -> Response6:
    return Foo2()


def f6_3() -> Response6:
    return Foo3()


def f6_4() -> Response6:
    return 3


def f6_5() -> Response6:
    # This should generate an error
    return None


class InnerA:
    pass


class InnerB:
    pass


T = TypeVar("T", bound=InnerA)


class A(Generic[T]):
    pass


class B:
    pass


U = Union[A[T], B]

a: U[InnerA]

# This should generate an error because InnerB is not
# compatible with the type bound to TypeVar T.
b: U[InnerB]


V = Union[A[T], T]

# This should generate an error because too many type
# arguments are provided.
c: V[InnerA, int]



# This sample covers the case where a metaclass (a class that derives
# from "type") is directly instantiated to create a new class,
# and that class is then used as a base class for another class.


class MyMeta(type):
    def do_something(self, p1: str, p2: int):
        pass


MyCustomClass = MyMeta("MyCustomClass", (object,), {})

reveal_type(MyCustomClass, expected_text="type[MyCustomClass]")


class DerivedCustomClass(MyCustomClass):
    pass


DerivedCustomClass.do_something("hi", 3)

# This should generate an error because the second
# argument is the wrong type.
DerivedCustomClass.do_something("hi", "no")

instance = DerivedCustomClass()



# This sample tests the type checker's expansion of constrained
# TypeVars within a union type.

from typing import Union, AnyStr, Sequence


def do_the_thing(param: Union[Sequence[AnyStr], AnyStr]) -> None:
    if isinstance(param, str):
        print(f"str: {param}")
        return

    if isinstance(param, bytes):
        print(f"bytes: {param!r}")
        return

    print(f"list:")
    for itm in param:
        print(f"  -> {itm}")


do_the_thing("a")
do_the_thing(b"b")
do_the_thing(["a", "b"])



# This sample tests the type checker's handling of derived specialized
# objects assigned to their parent class type (also specialized).

from typing import Generic, TypeVar

T = TypeVar("T", bound=float)


class Base1(Generic[T]):
    pass


class Derived1(Base1[T]):
    pass


val1: Base1[int] = Derived1[int]()


class Base2(Generic[T]):
    pass


class Derived2(Base2[float], Generic[T]):
    pass


val2_1: Base2[float] = Derived2[int]()

# This should generate an error because Derived2[int]
# isn't assignable to Base2[int].
val2_2: Base2[int] = Derived2[int]()



# This sample tests an auto-invariance case that involves recursive types.

from __future__ import annotations

from collections.abc import Callable
from dataclasses import dataclass
from typing import Concatenate, Protocol


@dataclass
class Node[T]:
    left: Node[T]
    right: Node[T]
    value: T


class MyPartial[**P, R]:
    def __init__(self, first: int, func: Callable[Concatenate[int, P], R]) -> None:
        self.first = first
        self.func = func

    def __call__(self, *args: P.args, **kwargs: P.kwargs) -> R: ...


class CallbackKeyed[*Ts](Protocol):
    def __call__(self, *args: *Ts, keyed: bool) -> tuple[*Ts]: ...


def invoke_keyed[*Ts](fn: CallbackKeyed[*Ts], *args: *Ts) -> tuple[*Ts]:
    return fn(*args, keyed=True)



# This sample tests the handling of TypeVar defaults with isinstance type
# narrowing.

from typing import Any, Generic, ParamSpec, TypeVar


P = ParamSpec("P", default=...)
R = TypeVar("R", default=Any)


class ParentA(Generic[P, R]): ...


class ChildA(ParentA[P, R]):
    pass


def func(x: ParentA[[int], int]):
    if isinstance(x, ChildA):
        reveal_type(x, expected_text="ChildA[(int), int]")
    else:
        reveal_type(x, expected_text="ParentA[(int), int]")



# This sample tests that protocol compatibility caching produces
# the correct result when the first example of protocol matching within
# the file does not require invariance enforcement but some later one
# does. The cached protocol compatibility cannot be used in this case.

from dataclasses import dataclass
from typing import Protocol


class P1(Protocol):
    x: int


class P2(Protocol):
    y: P1


@dataclass
class A:
    x: int


@dataclass
class B:
    y: A


y: P1 = A(3)

# This should generate an error.
x: P2 = B(A(3))

z: P1 = A(3)



# This sample tests the case where type[Self] is returned but Self
# is expected.

from typing import Self


class Foo:
    @classmethod
    def bar(cls) -> Self:
        # This should generate an error.
        return cls



# This sample tests the case where a function with a ParamSpec
# is assigned to another function with a Concatenate and a ParamSpec.

from typing import Any, Concatenate, Callable


class MyGeneric[**P0]:
    def __call__(self, *args: P0.args, **kwargs: P0.kwargs) -> Any: ...


def deco1[**P1](func: Callable[[Callable[P1, Any]], Any]) -> MyGeneric[P1]: ...


@deco1
def func1[**P2](func: Callable[Concatenate[int, P2], Any]): ...


reveal_type(func1, expected_text="MyGeneric[(int, **P2@func1)]")


v1: MyGeneric[[int]] = func1

# This should generate an error.
v2: MyGeneric[[int, int]] = func1



# This sample tests the case of a context manager within a try/except block.

from typing import ContextManager


def create_context() -> ContextManager[str]: ...


def possible_exception() -> None: ...


def func1():
    x: str | None = None
    ctx: str | None = None
    try:
        with create_context() as ctx:
            x = "0"
            possible_exception()
    except Exception:
        reveal_type(x, expected_text="Literal['0'] | None")
        reveal_type(ctx, expected_text="str | None")


def func2():
    ctx: str | None = None
    try:
        with create_context() as ctx:
            possible_exception()
            return
    except Exception:
        reveal_type(ctx, expected_text="str | None")



# This sample tests the handling of a property that's defined
# with a generic type for the "self" parameter.

from typing import TypeVar


_P = TypeVar("_P", bound=str)


class ClassA(str):
    @property
    def parent(self: _P) -> _P: ...


p = ClassA().parent
reveal_type(p, expected_text="ClassA")



# This sample tests the synthesis of "clear" and "popitem" within a closed
# TypedDict under certain circumstances.

from typing import NotRequired, Required, TypedDict
from typing_extensions import ReadOnly  # pyright: ignore[reportMissingModuleSource]


class TD1(TypedDict, total=False, extra_items=int):
    a: int


td1: TD1 = {"a": 1}

reveal_type(td1.clear, expected_text="() -> None")
reveal_type(td1.popitem, expected_text="() -> tuple[str, int]")
td1.clear()
td1.popitem()


class TD2(TypedDict, total=False, closed=True):
    a: str


td2: TD2 = {"a": "1"}

reveal_type(td2.clear, expected_text="() -> None")
reveal_type(td2.popitem, expected_text="() -> tuple[str, str]")
td2.clear()
td2.popitem()


class TD3(TypedDict, total=False, extra_items=ReadOnly[int]):
    a: int


td3: TD3 = {"a": 1}

# This should generate an error because extra_items is ReadOnly.
td3.clear()

# This should generate an error because extra_items is ReadOnly.
td3.popitem()


class TD4(TypedDict, closed=True):
    a: NotRequired[int]
    b: Required[int]


td4: TD4 = {"b": 1}

# This should generate an error because not all elements are NotRequired.
td4.clear()

# This should generate an error because not all elements are NotRequired.
td4.popitem()


class TD5(TypedDict, closed=True):
    a: NotRequired[ReadOnly[int]]


td5: TD5 = {"a": 1}

# This should generate an error because some elements are ReadOnly.
td5.clear()

# This should generate an error because some elements are ReadOnly.
td5.popitem()



# pyright: reportUnusedParameter=none

# The `self` or `cls` parameter of a method should not have a default value


class Foo:
    def instance_method1(self=42) -> None:
        pass

    def instance_method2(self=42, /, *, other=57) -> None:
        pass

    @classmethod
    def class_method(cls=42) -> None:
        pass

    @staticmethod
    def static_method(something=42) -> None:
        pass

    @staticmethod
    def static_method_kwonly(*, something=42) -> None:
        pass

    @staticmethod
    def static_method_pos_only(something=42, /) -> None:
        pass


def normal_function(x=42) -> None:
    pass


def typed_function(x: int = 42) -> None:
    pass



# This sample tests bidirectional type inference for calls where the expected
# type is a union.

from typing import Any, AnyStr, Iterable, Literal, TypeVar, overload

T = TypeVar("T")


class ItemBase: ...


class Item(ItemBase): ...


def gen_items() -> Iterable[Item]:
    return []


def make_list(val: Iterable[T]) -> list[T]:
    return list(val)


x: ItemBase | list[ItemBase | None] = make_list(gen_items())


@overload
def urlunsplit(components: Iterable[None]) -> Literal[b""]: ...


@overload
def urlunsplit(components: Iterable[AnyStr | None]) -> AnyStr: ...


def urlunsplit(components: Iterable[Any]) -> Any: ...


def func(url: str, candidates: list[Any]) -> str | None:
    return urlunsplit(candidates[0])



# This sample tests various forms of the 'with' statement.

from typing import Any, Generic, Optional, TypeVar, Self

_T1 = TypeVar("_T1")


class Class1(object):
    def __exit__(
        self,
        t: Optional[type] = None,
        exc: Optional[BaseException] = None,
        tb: Optional[Any] = None,
    ) -> bool:
        return True


class Class2(object):
    def __enter__(self):
        return 1

    def __exit__(
        self,
        t: Optional[type] = None,
        exc: Optional[BaseException] = None,
        tb: Optional[Any] = None,
    ) -> bool:
        return True


class Class3(object):
    def __enter__(self: _T1) -> _T1:
        return self

    def __exit__(
        self,
        t: Optional[type] = None,
        exc: Optional[BaseException] = None,
        tb: Optional[Any] = None,
    ) -> bool:
        return True


def requires_int(val: int):
    pass


def requires_class3(val: Class3):
    pass


def test1():
    a1 = Class1()

    # This should generate an error because Class1
    # does not implement an __enter__
    with a1 as foo:
        pass

    a2 = Class2()
    with a2 as foo:
        requires_int(foo)

    a3 = Class3()
    with a3 as foo:
        # This should generate an error because foo
        # should be of type Class3.
        requires_int(foo)

        requires_class3(foo)

    with a2 as foo2, a3 as foo3:
        requires_int(foo2)
        requires_class3(foo3)


class Class4:
    async def __aenter__(self: _T1) -> _T1:
        return self

    async def __aexit__(
        self,
        t: Optional[type] = None,
        exc: Optional[BaseException] = None,
        tb: Optional[Any] = None,
    ) -> bool:
        return True


async def test2():
    a1 = Class4()

    # This should generate two errors because Class4 does not
    # implement __enter__ or __exit__.
    with a1 as foo:
        pass

    async with a1 as foo:
        pass


class Class5(Generic[_T1]):
    async def __aenter__(self) -> Self:
        return self

    async def __aexit__(self, *args: Any) -> None:
        return None


class Class6(Class5[int]): ...


async def do():
    async with Class6() as f:
        reveal_type(f, expected_text="Class6")



# This sample tests the case where a constrained type variable
# includes a Literal[False] and Literal[True].

from typing import TypeVar, Generic, Literal

XOrY = TypeVar("XOrY", Literal[True], Literal[False])


class A(Generic[XOrY]):
    pass


class B(Generic[XOrY]):
    def __init__(self, a: A[XOrY]):
        self.a = a



# This sample tests a case of bidirectional type inference for calls
# that involves a union in the expected type.

# pyright: strict

from __future__ import annotations

from collections.abc import Callable
from typing import Generic, TypeVar

T_co = TypeVar("T_co", covariant=True)
E_co = TypeVar("E_co", covariant=True)
F = TypeVar("F")


class Ok(Generic[T_co]):
    def or_else(self, op: object) -> Ok[T_co]: ...


class Err(Generic[E_co]):
    def or_else(self, op: Callable[[E_co], Result[T_co, F]]) -> Result[T_co, F]: ...


Result = Ok[T_co] | Err[E_co]


def inner(func: Callable[[E_co], Err[F]], r: Result[T_co, E_co]) -> Result[T_co, F]:
    match r:
        case Ok():
            return r.or_else(func)
        case Err():
            return r.or_else(func)



# This sample verifies that enums are iterable and indexable.

import enum
from typing import Type


class Color(enum.Enum):
    RED = enum.auto()
    GREEN = enum.auto()


class Foo:
    _foo: Type[enum.Enum]

    def __init__(self):
        self._foo = Color

    def _print_foo(self):
        for f in self._foo:
            print(f)


def test_fn(color: Type[enum.Enum]):
    print(color["RED"])



# This sample tests that named tuple fields override abstract methods.

# pyright: reportIncompatibleVariableOverride=false

from abc import ABC, abstractmethod
from typing import NamedTuple


class ClassA(ABC):
    @property
    @abstractmethod
    def myproperty(self) -> str: ...


MixinB = NamedTuple("MixinB", [("myproperty", str)])


class ClassB(MixinB, ClassA):
    pass


ClassB(myproperty="myproperty")



# This sample tests the type checker's reporting of
# incompatible declared types.

from collections.abc import Callable


def a():
    pass


# This should generate an error for an incompatible assignment.
a = 1


# This should generate an error for an obscured type.
def b():
    pass


b: int = 1

# This should generate an error for an obscured type.
c: int = 1
c: float = 1.1

# This should generate two errors - one for an
# obscured type, the second for an incompatible assignment.
d: int = 2


def d():
    pass


class Foo:
    # This should generate an error because aa is redeclared.
    aa: int

    def aa(self):
        return 3


# This should generate two errors, one for each param.
def my_func(param1: int, param2):
    param1: int = 3
    param2: int = 4


# This should be fine because both declarations of 'e'
# use the same type.
# edit: actually no its not thats cringe
e: list[int]
e = [3]
e: list[int]


def register(fn: Callable[[], None]) -> None: ...


# These should be be fine because they use the "_" name.
@register
def _():
    print("Callback 1 called")


@register
def _():
    print("Callback 2 called")



# This sample tests the evaluation of a variable whose type is narrowed
# within a loop body.


class ClassA:
    def non_property(self) -> int: ...

    def do_stuff(self, x: int | None):
        while True:
            if x is not None:
                a = x
            else:
                a = self.non_property

            # This should generate an error because the type of "a"
            # is not compatible with a "-" operator.
            _ = a - 0



# This sample tests the handling of the Python 3.9
# TypeAlias feature as documented in PEP 613.

import sys
from typing import Type, TypeAlias as TA, Union, cast

type1: TA = Union[int, str]

type2: TA = "ClassA"


class ClassA:
    pass


not_a_type = "ClassA"


def requires_string(a: str):
    pass


requires_string(not_a_type)

# This should generate an error because type2 should
# not be interpreted as a string.
requires_string(type2)

# This should generate an error because the symbol
# is later declared as a TypeAlias.
my_type3 = int

# This should generate an error because it is obscured
# by another type alias declaration.
my_type3: "TA" = Union[int, str]

# This should generate an error because the symbol
# was previously declared as a TypeAlias.
my_type3: TA = int

# This should generate an error because the expression
# on the RHS evaluates to an object, not a class.
my_type4: TA = 3

# This should generate an error because the expression
# on the RHS evaluates to an object, not a class.
my_type5: TA = True

# This should generate an error because the expression
# on the RHS evaluates to an object, not a class.
my_type7: TA = list()

# Verify that variables with declarations (other than explicit TypeAlias)
# are not treated as a type alias.
SimpleAlias = int
ExplicitAlias: TA = int
SimpleNonAlias: Type[int] = int

if sys.version_info > (3, 9):
    reveal_type(SimpleAlias, expected_text="type[int]")
    reveal_type(ExplicitAlias, expected_text="type[int]")
    reveal_type(SimpleNonAlias, expected_text="type[int]")


class ClassB:
    my_type1: TA = int


def func1():
    # This should generate an error because type aliases are allowed
    # only in classes or modules.
    my_type1: TA = int


_Obj = cast(type[object], object)
# This should generate an error because _Obj is a variable,
# which isn't allowed in a TypeAlias statement.
Obj: TA = _Obj



# This sample tests the reportTypeCommentUsage diagnostic check.

from typing import Optional


# This should generate an error if reportTypeCommentUsage is enabled.
def func1a(a, b):
    # type: (int, str) -> str
    return ""


# This should generate an error if reportTypeCommentUsage is enabled.
def func1b(a, b):  # type: (Optional[str], int) -> str
    return ""


# This should generate an error if reportTypeCommentUsage is enabled.
def func1c(
    a,  # type: int
    b,  # type: str
):
    # type: (...) -> str
    return ""



# This sample tests the case where a local NoReturn call depends
# on the inferred type of a local variable.

from typing import NoReturn


class MyClass:
    def no_return(self) -> NoReturn: ...


def client_code() -> NoReturn:
    instance = MyClass()
    instance.no_return()



# This sample tests that classes whose metaclass implements a context
# manager work with the "with" statement.

from types import TracebackType


class ClassA(type):
    def __enter__(cls) -> "ClassA":
        print("Enter A")
        return cls

    def __exit__(
        cls, exc_typ: type[Exception], exc_val: Exception, exc_tbc: TracebackType
    ) -> None:
        print("Exit A")


class ClassB(metaclass=ClassA): ...


with ClassB as b:
    ...



# This sample tests the case where a generic class declaration refers
# to itself. This case should arguably be considered an error, but
# it does appear within the stdlib typeshed stubs (see os.scandir).

from os import DirEntry
from types import TracebackType
from typing import AnyStr, ContextManager, Iterator
from typing_extensions import Self  # pyright: ignore[reportMissingModuleSource]


class _ScandirIterator(
    Iterator[DirEntry[AnyStr]], ContextManager["_ScandirIterator[AnyStr]"]
):
    def __iter__(self) -> Self: ...

    def __next__(self) -> DirEntry[AnyStr]: ...

    def close(self) -> None: ...

    def __enter__(self) -> Self: ...

    def __exit__(
        self,
        __exc_type: type[BaseException] | None,
        __exc_value: BaseException | None,
        __traceback: TracebackType | None,
    ) -> bool | None: ...


def scandir(path: AnyStr) -> _ScandirIterator[AnyStr]: ...


def thing(value: AnyStr):
    with scandir(value) as it:
        for file in it:
            if isinstance(file.name, str):
                if file.name.endswith(".xml"):
                    ...
            elif isinstance(file.name, bytes):
                if file.name.endswith(b".xml"):
                    ...



# This sample tests that the type checker is properly synthesizing
# a constructor for a NewType.

from typing import NewType


UserId = NewType("UserId", int)

# This should generate an error because the constructor
# requires a single int.
var1 = UserId()

var2 = UserId(2)

# This should generate an error because the constructor
# requires a single int.
var3 = UserId("2")

# This should generate an error because the constructor
# requires a single int.
var4 = UserId(2, 3)


def require_user_id(a: UserId): ...


require_user_id(var2)

# this should generate an error.
require_user_id(2)

var5 = 4 + var2
var6 = var2 * 2



# This sample tests a complex TypeVar unification scenario.

from typing import Protocol, TypeVar

A = TypeVar("A", contravariant=True)
B = TypeVar("B", covariant=True)
T = TypeVar("T")
U = TypeVar("U")
V = TypeVar("V")


class Getter(Protocol[A, B]):
    def __call__(self, x: A, /) -> B: ...


class PolymorphicListItemGetter(Protocol):
    def __call__(self, l: list[T], /) -> T: ...


def compose(get1: Getter[T, U], get2: Getter[U, V]) -> Getter[T, V]: ...


class HasMethod(Protocol):
    @property
    def method(self) -> int: ...


def get_value(x: HasMethod) -> int: ...


def upcast(x: PolymorphicListItemGetter) -> Getter[list[HasMethod], HasMethod]:
    return x


def test(poly_getter: PolymorphicListItemGetter):
    compose(poly_getter, get_value)
    compose(upcast(poly_getter), get_value)



# This sample tests the case where an annotated variable in a loop
# is used as an index, an implicit argument for __getitem__.

# pyright: strict


def func1(nums: list[int], target: int) -> None:
    var = nums[0]
    while True:
        mid = var
        if nums[mid] == target:
            return
        if var:
            var = mid
        else:
            var = mid
